@@ -1,4 +1,4 @@
-/*  $Id: radius.c 8241 2008-12-17 18:26:44Z iulius $
+/*  $Id: radius.c 8469 2009-05-16 12:43:53Z iulius $
 **
 **  Authenticate a user against a remote radius server.
 */
@@ -20,9 +20,9 @@
 #include "inn/innconf.h"
 #include "inn/md5.h"
 #include "inn/messages.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 #include "nntp.h"
-#include "paths.h"
+#include "inn/paths.h"
 #include "conffile.h"
 
 #include "libauth.h"
@@ -36,7 +36,7 @@ typedef struct _auth_req {
     unsigned char	id;
     unsigned short	length;
     unsigned char	vector[AUTH_VECTOR_LEN];
-    unsigned char	data[NNTP_STRLEN*2];
+    unsigned char	data[NNTP_MAXLEN_COMMAND*2];
     int			datalen;
 } auth_req;
 
@@ -74,18 +74,18 @@ typedef struct _sending_t {
 #define RADsource  18
 
 static CONFTOKEN radtoks[] = {
-  { RADlbrace,   "{" },
-  { RADrbrace,   "}" },
-  { RADserver,   "server" },
-  { RADhost,     "radhost:" },
-  { RADsecret,   "secret:" },
-  { RADport,     "radport:" },
-  { RADlochost,  "lochost:" },
-  { RADlocport,  "locport:" },
-  { RADprefix,   "prefix:" },
-  { RADsuffix,   "suffix:" },
-  { RADsource,   "ignore-source:" },
-  { 0, 0 }
+    { RADlbrace,        (char *) "{"                    },
+    { RADrbrace,        (char *) "}"                    },
+    { RADserver,        (char *) "server"               },
+    { RADhost,          (char *) "radhost:"             },
+    { RADsecret,        (char *) "secret:"              },
+    { RADport,          (char *) "radport:"             },
+    { RADlochost,       (char *) "lochost:"             },
+    { RADlocport,       (char *) "locport:"             },
+    { RADprefix,        (char *) "prefix:"              },
+    { RADsuffix,        (char *) "suffix:"              },
+    { RADsource,        (char *) "ignore-source:"       },
+    { 0,                NULL                            }
 };
 
 static rad_config_t *get_radconf(void)
@@ -535,7 +535,7 @@ int main(int argc, char *argv[])
     if (argc != optind)
       exit(2);
     if (!havefile) {
-      radius_config = concatpath(innconf->pathetc, _PATH_RADIUS_CONFIG);
+      radius_config = concatpath(innconf->pathetc, INN_PATH_RADIUS_CONFIG);
       read_config(radius_config, &radconfig);
   
       free(radius_config);
@@ -560,6 +560,6 @@ int main(int argc, char *argv[])
             retval);
 
     /* radius password matches! */
-    printf("User:%s\n", authinfo->username);
+    print_user(authinfo->username);
     exit(0);
 }
