@@ -1,4 +1,4 @@
-##  $Id: newgroup.pl 7743 2008-04-06 10:04:43Z iulius $
+##  $Id: newgroup.pl 8404 2009-04-04 10:54:53Z iulius $
 ##
 ##  newgroup control message handler.
 ##
@@ -19,27 +19,68 @@ use strict;
 
 sub control_newgroup {
     my ($par, $sender, $replyto, $site, $action, $log, $approved,
-        $headers, $body) = @_;
+        $article, $charset_from, $charset_to) = @_;
     my ($groupname, $modflag) = @$par;
 
+    my $head = $article->head;
+    my @headers = split(/\r?\n/, $head->stringify);
+    my @fullbody = split(/\r?\n/, $article->stringify_body);
+    my (@body, $part, $part_head);
+    my $mimegroupinfo = 0;
+
+    my $charset_message;
+    if (defined $head->mime_attr('Content-Type.charset')) {
+        $charset_message = $head->mime_attr('Content-Type.charset');
+    }
+
+    # Check if it is a multipart message.  The body is restricted to
+    # the application/news-groupinfo part, if any.
+    if ($article->parts > 0) {
+        foreach $part ($article->parts) {
+            $part_head = $part->head;
+
+            if ($part_head->mime_type eq 'application/news-groupinfo') {
+                @body = split(/\r?\n/, $part->stringify_body);
+                if (defined $part_head->mime_attr('Content-Type.charset')) {
+                    $charset_message = $part_head->mime_attr('Content-Type.charset');
+                }
+                $mimegroupinfo = 1;
+            }
+        }
+    }
+
+    # The newgroup control message can be an application/news-groupinfo
+    # entity itself.
+    if ($head->mime_type eq 'application/news-groupinfo') {
+        @body = @fullbody;
+        $mimegroupinfo = 1;
+    }
+
+    @body = @fullbody if not $mimegroupinfo;
+
+    # Find the right charset if absent or forced by control.ctl.
+    foreach (@$charset_from) {
+        my ($group, $charset) = split /:/;
+        if ($groupname =~ /$group/) {
+            if (not defined $charset_message or $charset =~ /=force/) {
+                $charset_message = $charset;
+                $charset_message =~ s/\^(.+)\$/$1/;
+                $charset_message =~ s/\\//g;
+                $charset_message =~ s/=force//;
+            }
+            last;
+        }
+    }
+    if (not defined $charset_message
+        or not defined Encode::find_encoding($charset_message)) {
+        $charset_message = "cp1252";  # Default charset, when undefined.
+    }
+
     $modflag ||= '';
     my $modcmd = $modflag eq 'moderated' ? 'm' : 'y';
 
-    my $errmsg;
-    $errmsg= local_checkgroupname($groupname) if defined &local_checkgroupname;
-    if ($errmsg) {
-        $errmsg = checkgroupname($groupname) if $errmsg eq 'DONE';
-
-        if ($log) {
-            logger($log, "skipping newgroup ($errmsg)", $headers, $body);
-        } else {
-            logmsg("skipping newgroup ($errmsg)");
-        }
-        return;
-    }
-
     # Scan active to see what sort of change we are making.
-    open(ACTIVE, $inn::active) or logdie("Cannot open $inn::active: $!");
+    open(ACTIVE, $INN::Config::active) or logdie("Cannot open $INN::Config::active: $!");
     my @oldgroup;
     while (<ACTIVE>) {
         next unless /^(\Q$groupname\E)\s\d+\s\d+\s(\w)/;
@@ -47,16 +88,18 @@ sub control_newgroup {
         last;
     }
     close ACTIVE;
-    
+
     my $status;
-    my $ngdesc = 'No description.';
-    my $olddesc = '';    
+    my $ngdesc = '';
+    my $olddesc = '';
     my $ngname = $groupname;
 
     # If there is a tag line, search whether the description has changed.
     my $found = 0;
     my $ngline = '';
-    foreach (@$body) {
+    foreach (@body) {
+        $found = 1 if (($mimegroupinfo)
+                       and ($_ !~ /^For your newsgroups file:\s*$/));
         if ($found) {
             # It is the line which contains the description.
             $ngline = $_;
@@ -64,17 +107,13 @@ sub control_newgroup {
         }
         $found = 1 if $_ =~ /^For your newsgroups file:\s*$/;
     }
-    
+
     if ($found) {
       ($ngname, $ngdesc) = split(/\s+/, $ngline, 2);
-      if ($ngdesc) {
-          $ngdesc =~ s/\s+$//;
-          $ngdesc =~ s/\s+\(moderated\)\s*$//i;
-          $ngdesc .= ' (Moderated)' if $modflag eq 'moderated';
-      }
+
       # Scan newsgroups to see the previous description, if any.
-      open(NEWSGROUPS, $inn::newsgroups)
-          or logdie("Cannot open $inn::newsgroups: $!");
+      open(NEWSGROUPS, $INN::Config::newsgroups)
+          or logdie("Cannot open $INN::Config::newsgroups: $!");
       while (<NEWSGROUPS>) {
           if (/^\Q$groupname\E\s+(.*)/) {
               $olddesc = $1;
@@ -84,6 +123,9 @@ sub control_newgroup {
       close NEWSGROUPS;
     }
 
+    # Properly encode the newsgroup description.
+    Encode::from_to($ngdesc, $charset_message, $charset_to);
+
     if (@oldgroup) {
         if ($oldgroup[3] eq 'm' and $modflag ne 'moderated') {
             $status = 'be made unmoderated';
@@ -102,6 +144,31 @@ sub control_newgroup {
         $status = 'be created';
     }
 
+    # Check whether the group name and the description are not junk.
+    my $errmsg;
+    if (defined &local_checkgroupname) {
+        $errmsg = local_checkgroupname($groupname);
+    } else {
+        $errmsg = checkgroupname($groupname);
+    }
+    if (! $errmsg) {
+        if (defined &local_checkdescription) {
+            $errmsg = local_checkdescription($ngdesc, $modcmd);
+        } else {
+            $errmsg = checkdescription($ngdesc, $modcmd);
+        }
+    }
+    if ($errmsg) {
+        if ($log) {
+            logger($log, "skipping newgroup $groupname $modcmd"
+                   . " $sender (would $status): $errmsg", $article);
+        } else {
+            logmsg("skipping newgroup $groupname $modcmd $sender"
+                   . " (would $status): $errmsg");
+        }
+        return;
+    }
+
     if ($action eq 'mail' and $status !~ /(no change|unapproved)/) {
         my $mail = sendmail("newgroup $groupname $modcmd $sender");
         print $mail <<END;
@@ -109,22 +176,22 @@ $sender asks for $groupname
 to $status.
 
 If this is acceptable, type:
-  $inn::newsbin/ctlinnd newgroup $groupname $modcmd $sender
+  $INN::Config::newsbin/ctlinnd newgroup $groupname $modcmd $sender
 
 And do not forget to update the corresponding description in your
-newsgroups file.
+$INN::Config::pathdb/newsgroups file.
 
 The control message follows:
 
 END
-        print $mail map { s/^~/~~/; "$_\n" } @$headers;
+        print $mail map { s/^~/~~/; "$_\n" } @headers;
         print $mail "\n";
-        print $mail map { s/^~/~~/; "$_\n" } @$body;
+        print $mail map { s/^~/~~/; "$_\n" } @fullbody;
         close $mail or logdie("Cannot send mail: $!");
     } elsif ($action eq 'log') {
         if ($log) {
             logger($log, "skipping newgroup $groupname $modcmd"
-                . " $sender (would $status)", $headers, $body);
+                . " $sender (would $status)", $article);
         } else {
             logmsg("skipping newgroup $groupname $modcmd $sender"
                 . " (would $status)");
@@ -138,10 +205,10 @@ END
             # We know the description has changed.
             update_desc($ngname, $ngdesc) if $ngdesc and $ngname eq $groupname;
         }
-        
-        if ($log ne 'mail' or $status ne 'no change') {
+
+        if ($log) {
             logger($log, "newgroup $groupname $modcmd $status $sender",
-                   $headers, $body) if $log;
+                   $article) if ($log ne 'mail' or $status ne 'no change');
         }
     }
     return;
@@ -149,10 +216,10 @@ END
 
 sub update_desc {
     my ($name, $desc) = @_;
-    shlock("$inn::locks/LOCK.newsgroups");
-    my $tempfile = "$inn::newsgroups.$$";
-    open(NEWSGROUPS, $inn::newsgroups)
-        or logdie("Cannot open $inn::newsgroups: $!");
+    shlock("$INN::Config::locks/LOCK.newsgroups");
+    my $tempfile = "$INN::Config::newsgroups.$$";
+    open(NEWSGROUPS, $INN::Config::newsgroups)
+        or logdie("Cannot open $INN::Config::newsgroups: $!");
     open(TEMPFILE, ">$tempfile") or logdie("Cannot open $tempfile: $!");
     while (<NEWSGROUPS>) {
         next if (/^\Q$name\E\s+(.*)/);
@@ -168,9 +235,9 @@ sub update_desc {
     }
     close TEMPFILE;
     close NEWSGROUPS;
-    rename($tempfile, $inn::newsgroups)
+    rename($tempfile, $INN::Config::newsgroups)
         or logdie("Cannot rename $tempfile: $!");
-    unlink("$inn::locks/LOCK.newsgroups", $tempfile);
+    unlink("$INN::Config::locks/LOCK.newsgroups", $tempfile);
 }
 
 # Check the group name.  This is partially derived from C News.
@@ -179,36 +246,59 @@ sub update_desc {
 sub checkgroupname {
     local $_ = shift;
 
-    # whole-name checking
-    return 'Empty group name' if /^$/;
+    # Whole-name checking.
+    return 'Empty group name' if (! $_);
     return 'Whitespace in group name' if /\s/;
-    return 'Unsafe group name' if /[\`\/:;]/;
+#    return 'Unsafe group name' if /[\`\/:;]/;
     return 'Bad dots in group name' if /^\./ or /\.$/ or /\.\./;
 #    return 'Group name does not begin/end with alphanumeric'
 #        if (/^[a-zA-Z0-9].+[a-zA-Z0-9]$/;
-    return 'Group name begins in control., junk. or to.' if /^(?:control|junk|to)\./;
+    return 'Group name begins in control., example. or to.'
+        if /^(?:control|example|to)\./;
+    return 'Group name is control, example, junk, poster or to'
+        if /^(?:control|example|junk|poster|to)$/;
 #    return 'Group name too long' if length $_ > 128;
 
     my @components = split(/\./);
-    # prevent alt.a.b.c.d.e.f.g.w.x.y.z...
+    # Prevent alt.a.b.c.d.e.f.g.w.x.y.z...
     return 'Too many components' if $#components > 9;
 
-    # per-component checking
+    # Per-component checking.
     for (my $i = 0; $i <= $#components; $i++) {
         local $_ = $components[$i];
-        return 'all-numeric name component' if /^[0-9]+$/;
-#        return 'name component starts with non-alphanumeric' if /^[a-zA-Z0-9]/;
-#        return 'name component does not contain letter' if not /[a-zA-Z]/;
+        return 'All-numeric name component' if /^[0-9]+$/;
+#        return 'Name component starts with non-alphanumeric' if /^[a-zA-Z0-9]/;
+#        return 'Name component does not contain letter' if not /[a-zA-Z]/;
         return "`all' or `ctl' used as name component" if /^(?:all|ctl)$/;
-#        return 'name component longer than 30 characters' if length $_ > 30;
-#        return 'uppercase letter(s) in name' if /[A-Z]/;
-        return 'illegal character(s) in name' if /[^a-z0-9+_\-.]/;
-        # sigh, c++ etc must be allowed
-        return 'repeated punctuation in name' if /--|__|\+\+./;
-#        return 'repeated component(s) in name' if ($i + 2 <= $#components
+#        return 'Name component longer than 30 characters' if length $_ > 30;
+        return 'Uppercase letter(s) in name' if /[A-Z]/;
+        return 'Illegal character(s) in name' if /[^a-z0-9+_\-]/;
+        # Sigh, c++ etc. must be allowed.
+#        return 'Repeated punctuation in name' if /--|__|\+\+./;
+#        return 'Repeated component(s) in name' if ($i + 2 <= $#components
 #            and $_ eq $components[$i + 1] and $_ eq $components[$i + 2]);
     }
     return '';
 }
 
+
+# Check the description.
+sub checkdescription {
+    my ($desc, $flag) = @_;
+
+    # Whole-name checking.
+    return 'Empty description' if (! $desc);
+
+    return 'Moderation status mismatch'
+        if ($desc =~ / \(Moderated\)$/) and $flag eq 'y';
+
+    return 'Moderation status mismatch'
+        if ($desc !~ / \(Moderated\)$/) and $flag eq 'm';
+
+    return 'Reserved "(Moderated)" substring used'
+        if ($desc =~ /\(Moderated\).+$/);
+
+    return '';
+}
+
 1;
