@@ -1,4 +1,4 @@
-/*  $Id: hisv6.c,v 1.11 2003/04/23 12:33:32 alexk Exp $
+/*  $Id: hisv6.c 8575 2009-08-18 13:53:54Z iulius $
 **
 **  History v6 implementation against the history API.
 **
@@ -45,11 +45,12 @@
 #include "hisinterface.h"
 #include "hisv6.h"
 #include "hisv6-private.h"
-#include "dbz.h"
+#include "inn/dbz.h"
+#include "inn/innconf.h"
 #include "inn/timer.h"
 #include "inn/qio.h"
 #include "inn/sequence.h"
-#include "inndcomm.h"
+#include "inn/inndcomm.h"
 
 /*
 **  because we can only have one open dbz per process, we keep a
@@ -96,7 +97,8 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
 		 time_t *arrived, time_t *posted, time_t *expires,
 		 TOKEN *token)
 {
-    char *p = (char *)line;
+    const char *p = line;
+    char *end;
     unsigned long l;
     int r = 0;
 
@@ -121,7 +123,8 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
     }
 
     /* parse the arrived field */
-    l = strtoul(p + 1, &p, 10);
+    l = strtoul(p + 1, &end, 10);
+    p = end;
     if (l == ULONG_MAX) {
 	*error = "arrived timestamp out of range";
 	return -1;
@@ -143,7 +146,8 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
 	    if (expires)
 		*expires = 0;
 	} else {
-	    l = strtoul(p, &p, 10);
+	    l = strtoul(p, &end, 10);
+            p = end;
 	    if (l == ULONG_MAX) {
 		*error = "expires timestamp out of range";
 		return -1;
@@ -159,7 +163,8 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
 		*posted = 0;
 	} else {
 	    ++p;
-	    l = strtoul(p, &p, 10);
+	    l = strtoul(p, &end, 10);
+            p = end;
 	    if (l == ULONG_MAX) {
 		*error = "posted timestamp out of range";
 		return -1;
@@ -259,6 +264,7 @@ hisv6_closefiles(struct hisv6 *h)
 	    r = false;
 	}
 	h->writefp = NULL;
+        h->offset = 0;
     }
 
     h->nextcheck = 0;
@@ -286,7 +292,7 @@ hisv6_reopen(struct hisv6 *h)
 	if (h->flags & HIS_CREAT)
 	    mode = "w";
 	else
-	    mode = "a";
+	    mode = "r+";
 	if ((h->writefp = Fopen(h->histpath, mode, INND_HISTORY)) == NULL) {
 	    hisv6_seterror(h, concat("can't fopen history ",
 				      h->histpath, " ",
@@ -294,12 +300,6 @@ hisv6_reopen(struct hisv6 *h)
 	    hisv6_closefiles(h);
 	    goto fail;
 	}
-
-	/* fseeko to the end of file because the result of ftello() is
-	   undefined for files freopen()-ed in append mode according
-	   to POSIX 1003.1.  ftello() is used later on to determine a
-	   new article's offset in the history file. Fopen() uses
-	   freopen() internally. */
 	if (fseeko(h->writefp, 0, SEEK_END) == -1) {
 	    hisv6_seterror(h, concat("can't fseek to end of ",
 				      h->histpath, " ",
@@ -307,6 +307,13 @@ hisv6_reopen(struct hisv6 *h)
 	    hisv6_closefiles(h);
 	    goto fail;
 	}
+        h->offset = ftello(h->writefp);
+	if (h->offset == -1) {
+	    hisv6_seterror(h, concat("can't ftello ", h->histpath, " ",
+				      strerror(errno), NULL));
+	    hisv6_closefiles(h);
+	    goto fail;
+	}
 	close_on_exec(fileno(h->writefp), true);
     }
 
@@ -348,6 +355,17 @@ hisv6_reopen(struct hisv6 *h)
 	    /*opt.pag_incore = INCORE_NO;*/
 	    opt.pag_incore = (h->flags & HIS_MMAP) ? INCORE_MMAP : INCORE_NO;
 	    opt.exists_incore = (h->flags & HIS_MMAP) ? INCORE_MMAP : INCORE_NO;
+
+# if defined(MMAP_NEEDS_MSYNC) && INND_DBZINCORE == 1
+	    /* Systems that have MMAP_NEEDS_MSYNC defined will have their
+	       on-disk copies out of sync with the mmap'ed copies most of
+	       the time.  So if innd is using INCORE_MMAP, then we force
+	       everything else to use it, too (unless we're on NFS) */
+	    if(!innconf->nfsreader) {
+		opt.pag_incore = INCORE_MMAP;
+		opt.exists_incore = INCORE_MMAP;
+	    }
+# endif
 #endif
 	}
 	dbzsetoptions(opt);
@@ -450,6 +468,7 @@ hisv6_new(const char *path, int flags, struct history *history)
     h->histpath = path ? xstrdup(path) : NULL;
     h->flags = flags;
     h->writefp = NULL;
+    h->offset = 0;
     h->history = history;
     h->readfd = -1;
     h->nextcheck = 0;
@@ -703,10 +722,11 @@ hisv6_check(void *history, const char *key)
 
 
 /*
-**  format a history line; `s' should be of at least HISV6_MAXLINE+1
-**  characters (to allow for the \0).
+**  Format a history line.  s should hold at least HISV6_MAXLINE + 1
+**  characters (to allow for the nul).  Returns the length of the data
+**  written, 0 if there was some error or if the data was too long to write.
 */
-static bool
+static int
 hisv6_formatline(char *s, const HASH *hash, time_t arrived,
 		  time_t posted, time_t expires, const TOKEN *token)
 {
@@ -714,9 +734,20 @@ hisv6_formatline(char *s, const HASH *hash, time_t arrived,
     const char *hashtext = HashToText(*hash);
 
     if (token == NULL) {
-	i = snprintf(s, HISV6_MAXLINE, "[%s]%c%lu%c%c\n",
-		     hashtext, HISV6_FIELDSEP,
-		     (unsigned long)arrived, HISV6_SUBFIELDSEP, HISV6_NOEXP);
+        /* Only a line to remember an article.  We keep its arrival
+         * and posting time. */
+        if (posted <= 0) {
+            i = snprintf(s, HISV6_MAXLINE, "[%s]%c%lu%c%c\n",
+                         hashtext, HISV6_FIELDSEP,
+                         (unsigned long)arrived, HISV6_SUBFIELDSEP,
+                         HISV6_NOEXP);
+        } else {
+            i = snprintf(s, HISV6_MAXLINE, "[%s]%c%lu%c%c%c%lu\n",
+                         hashtext, HISV6_FIELDSEP,
+                         (unsigned long)arrived, HISV6_SUBFIELDSEP,
+                         HISV6_NOEXP, HISV6_SUBFIELDSEP,
+                         (unsigned long)posted);
+        }
     } else {
 	const char *texttok;
 
@@ -738,8 +769,8 @@ hisv6_formatline(char *s, const HASH *hash, time_t arrived,
 	}
     }
     if (i < 0 || i >= HISV6_MAXLINE)
-	return false;
-    return true;
+	return 0;
+    return i;
 }
 
 
@@ -793,9 +824,8 @@ static bool
 hisv6_writeline(struct hisv6 *h, const HASH *hash, time_t arrived,
 		time_t posted, time_t expires, const TOKEN *token)
 {
-    int i;
     bool r;
-    off_t offset;
+    size_t i, length;
     char hisline[HISV6_MAXLINE + 1];
     char location[HISV6_MAX_LOCATION];
 
@@ -811,26 +841,31 @@ hisv6_writeline(struct hisv6 *h, const HASH *hash, time_t arrived,
 	return false;
     }
 
-    if (hisv6_formatline(hisline, hash, arrived, posted, expires,
-			  token) != true) {
+    length = hisv6_formatline(hisline, hash, arrived, posted, expires, token);
+    if (length == 0) {
 	hisv6_seterror(h, concat("error formatting history line ",
 				  h->histpath, NULL));
 	return false;
     }	
 
-    offset = ftello(h->writefp);
-    i = fputs(hisline, h->writefp);
-    if (i == EOF ||
-	(!(h->flags & HIS_INCORE) && fflush(h->writefp) == EOF)) {
-	hisv6_errloc(location, (size_t)-1, offset);
-	/* The history line is now an orphan... */
+    i = fwrite(hisline, 1, length, h->writefp);
+
+    /* If the write failed, the history line is now an orphan.  Attempt to
+       rewind the write pointer to our offset to avoid leaving behind a
+       partial write and desyncing the offset from our file position. */
+    if (i < length ||
+        (!(h->flags & HIS_INCORE) && fflush(h->writefp) == EOF)) {
+	hisv6_errloc(location, (size_t)-1, h->offset);
 	hisv6_seterror(h, concat("can't write history ", h->histpath,
 				  location, " ", strerror(errno), NULL));
+        if (fseeko(h->writefp, h->offset, SEEK_SET) == -1)
+            h->offset += i;
 	r = false;
 	goto fail;
     }
 
-    r = hisv6_writedbz(h, hash, offset);
+    r = hisv6_writedbz(h, hash, h->offset);
+    h->offset += length;     /* increment regardless of error from writedbz */
  fail:
     return r;
 }
@@ -857,10 +892,11 @@ hisv6_write(void *history, const char *key, time_t arrived,
 
 
 /*
-**  remember a history entry, key, with arrival time arrived.
+**  Remember a history entry, key, with arrival time, and also
+**  posting time if known.
 */
 bool
-hisv6_remember(void *history, const char *key, time_t arrived)
+hisv6_remember(void *history, const char *key, time_t arrived, time_t posted)
 {
     struct hisv6 *h = history;
     HASH hash;
@@ -868,7 +904,7 @@ hisv6_remember(void *history, const char *key, time_t arrived)
 
     his_logger("HISwrite begin", S_HISwrite);
     hash = HashMessageID(key);
-    r = hisv6_writeline(h, &hash, arrived, 0, 0, NULL);
+    r = hisv6_writeline(h, &hash, arrived, posted, 0, NULL);
     his_logger("HISwrite end", S_HISwrite);
     return r;
 }
@@ -903,7 +939,7 @@ hisv6_replace(void *history, const char *key, time_t arrived,
 	char new[HISV6_MAXLINE + 1];
 
 	if (hisv6_formatline(new, &hash, arrived, posted, expires,
-			      token) != true) {
+                             token) == 0) {
  	    hisv6_seterror(h, concat("error formatting history line ",
 				      h->histpath, NULL));
 	    r = false;
@@ -912,7 +948,9 @@ hisv6_replace(void *history, const char *key, time_t arrived,
 
 	    oldlen = strlen(old);
 	    newlen = strlen(new);
-	    if (new > old) {
+	    if (new[newlen - 1] == '\n')
+                newlen--;
+	    if (newlen > oldlen) {
 		hisv6_seterror(h, concat("new history line too long ",
 					  h->histpath, NULL));
 		r = false;
@@ -920,12 +958,12 @@ hisv6_replace(void *history, const char *key, time_t arrived,
 		ssize_t n;
 
 		/* space fill any excess in the tail of new */
-		memset(new + oldlen, ' ', oldlen - newlen);
+		memset(new + newlen, ' ', oldlen - newlen);
 
 		do {
 		    n = pwrite(fileno(h->writefp), new, oldlen, offset);
 		} while (n == -1 && errno == EINTR);
-		if (n != oldlen) {
+		if ((size_t) n != oldlen) {
 		    char location[HISV6_MAX_LOCATION];
 
 		    hisv6_errloc(location, (size_t)-1, offset);
@@ -955,7 +993,7 @@ hisv6_traverse(struct hisv6 *h, struct hisv6_walkstate *cookie,
 				time_t, time_t, time_t,
 				const TOKEN *))
 {
-    bool r;
+    bool r = false;
     QIOSTATE *qp;
     void *p;
     size_t line;
@@ -1114,18 +1152,24 @@ hisv6_expirecb(struct hisv6 *h, void *cookie, const HASH *hash,
 	    keep = (*hiscookie->cb.expire)(hiscookie->cookie,
 					   arrived, posted, expires,
 					   t);
-	    /* if the callback returns true, we should keep the
-	       token for the time being, else we just remember
-	       it */
+	    /* If the callback returns true, we should keep the
+	     * token for the time being, else we just remember
+	     * it. */
 	    if (keep == false) {
 		t = NULL;
-		posted = expires = 0;
+		expires = 0;
 	    }
 	} else {
 	    t = NULL;
 	}
+        /* When t is NULL (no token), the message-ID is removed from
+         * history when the posting time of the article is older than
+         * threshold, as set by the /remember/ line in expire.ctl.
+         * We keep the check for the arrival time because some entries
+         * might not have one. */
 	if (hiscookie->new &&
-	    (t != NULL || arrived >= hiscookie->threshold)) {
+	    (t != NULL || posted >= hiscookie->threshold
+             || (posted <= 0 && arrived >= hiscookie->threshold))) {
 	    r = hisv6_writeline(hiscookie->new, hash,
 				 arrived, posted, expires, t);
 	}
@@ -1215,7 +1259,7 @@ hisv6_expire(void *history, const char *path, const char *reason,
 	     bool (*exists)(void *, time_t, time_t, time_t, TOKEN *))
 {
     struct hisv6 *h = history, *hnew = NULL;
-    const char *nhistory = NULL;
+    char *nhistory = NULL;
     dbzoptions opt;
     bool r;
     struct hisv6_walkstate hiscookie;
@@ -1243,7 +1287,8 @@ hisv6_expire(void *history, const char *path, const char *reason,
 	    nhistory = concat(h->histpath, ".n", NULL);
 	}
 
-	hnew = hisv6_new(nhistory, HIS_RDWR | HIS_INCORE, h->history);
+	hnew = hisv6_new(nhistory, HIS_CREAT | HIS_RDWR | HIS_INCORE,
+                         h->history);
 	if (!hisv6_reopen(hnew)) {
 	    hisv6_dispose(hnew);
 	    hnew = NULL;
@@ -1337,7 +1382,7 @@ hisv6_expire(void *history, const char *path, const char *reason,
     if (hnew && !hisv6_dispose(hnew))
 	r = false;
     if (nhistory && nhistory != path)
-	free((char *)nhistory);
+	free(nhistory);
     if (r == false && hiscookie.paused)
 	ICCgo(reason);
     return r;
