@@ -1,4 +1,4 @@
-/*  $Revision: 1.5 $
+/*  $Revision: 6796 $
 **
 **  Data structures, functions and cetera used for config file parsing.
 */
@@ -25,10 +25,10 @@ typedef struct {
 
 extern char CONFerror[];
 
-extern CONFFILE *CONFfopen(char*);
-extern void CONFfclose(CONFFILE*);
+extern CONFFILE *CONFfopen(const char *);
+extern void CONFfclose(CONFFILE *);
 
-extern CONFTOKEN *CONFgettoken(CONFTOKEN*, CONFFILE*);
+extern CONFTOKEN *CONFgettoken(CONFTOKEN *, CONFFILE *);
 
 #ifdef __cplusplus
 }
