@@ -1,4 +1,4 @@
-/*  $Id: wire.h 8788 2009-11-15 09:27:11Z iulius $
+/*  $Id: wire.h 7366 2005-07-03 19:14:16Z eagle $
 **
 **  Wire format article utilities.
 **
@@ -28,11 +28,13 @@ char *wire_nextline(const char *, const char *end);
 
 /* Given a pointer to the start of an article and the name of a header, find
    the beginning of the value of the given header (the returned pointer will
-   be after the name of the header, and also any initial whitespace if specified
-   by the stripspaces argument).  Headers whose only content is whitespace are
-   ignored when whitespaces are stripped.  If the header isn't found, returns
-   NULL. */
-char *wire_findheader(const char *article, size_t, const char *header, bool stripspaces);
+   be after the name of the header and any initial whitespace).  Headers whose
+   only content is whitespace are ignored.  If the header isn't found, returns
+   NULL.
+
+   WARNING: This function does not comply with RFC 2822's idea of header
+   content, particularly in its skipping of initial whitespace. */
+char *wire_findheader(const char *article, size_t, const char *header);
 
 /* Given a pointer inside a header's value and a pointer to the end of the
    article, returns a pointer to the end of the header value (the \n at the
