@@ -1,4 +1,4 @@
-/*  $Id: art.c 8469 2009-05-16 12:43:53Z iulius $
+/*  $Id: art.c 8657 2009-10-10 07:58:53Z iulius $
 **
 **  Article-processing.
 */
@@ -400,23 +400,35 @@ ARTlogreject(CHANNEL *cp, const char *text)
 
     /* Set up the headers that we want to use.  We only need to parse the path
        on rejections if logipaddr is false or we can't find a good host. */
-    if (innconf->logipaddr && cp->Address.ss_family != 0)
-        data->Feedsite = RChostname(cp);
-    else {
+    if (innconf->logipaddr) {
+        if (cp->Address.ss_family != 0) {
+            data->Feedsite = RChostname(cp);
+        } else {
+            data->Feedsite = "localhost";
+        }
+    } else {
         if (HDR_FOUND(HDR__PATH)) {
             HDR_PARSE_START(HDR__PATH);
             hopcount =
                 ARTparsepath(HDR(HDR__PATH), HDR_LEN(HDR__PATH), &data->Path);
             HDR_PARSE_END(HDR__PATH);
             hops = data->Path.List;
-            if (hopcount > 0 && hops != NULL && hops[0] != NULL)
+            if (hopcount > 0 && hops != NULL && hops[0] != NULL) {
                 data->Feedsite = hops[0];
-            else
+            } else {
                 data->Feedsite = "localhost";
-        } else if (cp->Address.ss_family != 0) {
-            data->Feedsite = RChostname(cp);
+            }
+        } else {
+            if (cp->Address.ss_family != 0) {
+                data->Feedsite = RChostname(cp);
+            } else {
+                data->Feedsite = "localhost";
+            }
         }
     }
+
+    data->FeedsiteLength = strlen(data->Feedsite);
+
     ARTlog(data, ART_REJECT, text != NULL ? text : cp->Error);
 
     if (HDR_FOUND(HDR__MESSAGE_ID))
@@ -804,6 +816,7 @@ ARTprepare(CHANNEL *cp)
   data->CurHeader = data->LastCRLF = data->Body = cp->Start;
   data->BytesHeader = NULL;
   data->Feedsite = "?";
+  data->FeedsiteLength = strlen(data->Feedsite);
   *cp->Error = '\0';
 }
 
@@ -856,7 +869,9 @@ ARTchecksize(CHANNEL *cp)
 	if (HDR_FOUND(HDR__MESSAGE_ID)) {
             HDR_PARSE_START(HDR__MESSAGE_ID);
             msgid = HDR(HDR__MESSAGE_ID);
-            if (!HIScheck(History, msgid) && !InndHisRemember(msgid))
+            /* The article posting time has not been parsed.  We cannot
+             * give it to InndHisRemember. */
+            if (!HIScheck(History, msgid) && !InndHisRemember(msgid, 0))
                 warn("SERVER cant write %s", msgid);
 	}
     }
@@ -911,7 +926,7 @@ ARTparseheader(CHANNEL *cp)
                 ARTerror(cp, "Header line too long (%lu bytes)", length);
 
             /* Be a little tricky here.  Normally, the headers end at the
-               first occurrance of \r\n\r\n, so since we've seen \r\n, we want
+               first occurrence of \r\n\r\n, so since we've seen \r\n, we want
                to advance i and then look to see if we have another one.  The
                exception is the degenerate case of an article with no headers.
                In that case, log an error and *don't* advance i so that we'll
@@ -1035,6 +1050,7 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
   TMRstart(TMR_ARTCLEAN);
   data->Arrived = Now.tv_sec;
   data->Expires = 0;
+  data->Posted = 0;
 
   /* replace trailing '\r\n' with '\0\n' of all system header to be handled
      easily by str*() functions */
@@ -1184,17 +1200,18 @@ ARTreject(Reject_type code, CHANNEL *cp)
 }
 
 /*
-**  Verify if a cancel message is valid.  If the user posting the cancel
-**  matches the user who posted the article, return the list of filenames
-**  otherwise return NULL.
+**  Verify if a cancel message is valid.  Unless at least one group in the 
+**  cancel message's Newsgroups: line can be found in the Newsgroups: line 
+**  of the article to be cancelled, the cancel is considered bogus and 
+**  false is returned.
 */
 static bool
 ARTcancelverify(const ARTDATA *data, const char *MessageID, TOKEN *token)
 {
-  const HDRCONTENT *hc = data->HdrContent;
   const char	*p;
   char		*q, *q1;
-  const char	*local, *poster;
+  char          **gp;
+  const char	*local;
   char		buff[SMBUF];
   ARTHANDLE	*art;
   bool		r;
@@ -1203,13 +1220,13 @@ ARTcancelverify(const ARTDATA *data, const char *MessageID, TOKEN *token)
     return false;
   if ((art = SMretrieve(*token, RETR_HEAD)) == NULL)
     return false;
-  local = wire_findheader(art->data, art->len, "Sender");
+
+  /* Copy Newsgroups: from article be to cancelled to q.
+   * Double-terminate q (sentinel). */
+  local = wire_findheader(art->data, art->len, "Newsgroups");
   if (local == NULL) {
-    local = wire_findheader(art->data, art->len, "From");
-    if (local == NULL) {
-      SMfreearticle(art);
-      return false;
-    }
+    SMfreearticle(art);
+    return false;
   }
   for (p = local; p < art->data + art->len; p++) {
     if (*p == '\r' || *p == '\n')
@@ -1219,30 +1236,37 @@ ARTcancelverify(const ARTDATA *data, const char *MessageID, TOKEN *token)
     SMfreearticle(art);
     return false;
   }
-  q = xmalloc(p - local + 1);
+  q = xmalloc(p - local + 2);
   memcpy(q, local, p - local);
   SMfreearticle(art);
   q[p - local] = '\0';
-  HeaderCleanFrom(q);
+  q[p - local + 1] = '\0';
 
-  /* Compare canonical forms. */
-  if (HDR_FOUND(HDR__SENDER))
-    poster = HDR(HDR__SENDER);
-  else
-    poster = HDR(HDR__FROM);
-  q1 = xstrdup(poster);
-  HeaderCleanFrom(q1);
-  if (strcmp(q, q1) != 0) {
-    r = false;
-    sprintf(buff, "\"%.50s\" wants to cancel %s by \"%.50s\"",
-      q1, MaxLength(MessageID, MessageID), q);
+  /* Replace separator ',' by '\0'. */
+  for (q1 = q; *q1; q1++) {
+    if (NG_ISSEP(*q1)) {
+      *q1 = '\0';
+    }
+  }
+
+  r = false;
+  for (gp = data->Newsgroups.List; *gp && !r; gp++) {
+    for (q1 = q; *q1; q1 += strlen(q1) + 1) {
+      if (strcmp(q1, *gp) == 0) {
+        r = true;
+        break;
+      }
+    }
+  }
+
+  free(q);
+
+  if (!r) {
+    sprintf(buff, "No matching newsgroups in cancel %s",
+            MaxLength(MessageID, MessageID));
     ARTlog(data, ART_REJECT, buff);
   }
-  else {
-    r = true;
-  }
-  free(q1);
-  free(q);
+      
   return r;
 }
 
@@ -1276,7 +1300,7 @@ ARTcancel(const ARTDATA *data, const char *MessageID, const bool Trusted)
       TMRstop(TMR_ARTCNCL);
       return;
     }
-    InndHisRemember(MessageID);
+    InndHisRemember(MessageID, data->Posted);
     snprintf(buff, sizeof(buff), "Cancelling %s",
              MaxLength(MessageID, MessageID));
     ARTlog(data, ART_CANC, buff);
@@ -1854,7 +1878,7 @@ ARTmakeoverview(CHANNEL *cp)
     j = hp - ARTheaders;
 
     /* If requested, generate keywords from the body of the article and patch
-       them into the apparent value of the Keywords header so that they make
+       them into the apparent value of the Keywords: header so that they make
        it into overview. */
     if (DO_KEYWORDS && innconf->keywords) {
       /* Ensure that there are Keywords: to shovel. */
@@ -1862,7 +1886,7 @@ ARTmakeoverview(CHANNEL *cp)
 	key_old_value  = HDR(HDR__KEYWORDS);
 	key_old_length = HDR_LEN(HDR__KEYWORDS);
 	KEYgenerate(&hc[HDR__KEYWORDS], cp->In.data + data->Body,
-                    key_old_value, key_old_length);
+                    cp->Next - data->Body, key_old_value, key_old_length);
       }
     }
 
@@ -1915,11 +1939,11 @@ ARTmakeoverview(CHANNEL *cp)
     /* Patch the old keywords back in. */
     if (DO_KEYWORDS && innconf->keywords) {
       if (key_old_value) {
-	if (hc->Value)
-	  free(hc->Value);		/* malloc'd within */
-	hc->Value  = key_old_value;
-	hc->Length = key_old_length;
-	key_old_value = NULL;
+        if (hc[HDR__KEYWORDS].Value)
+          free(hc[HDR__KEYWORDS].Value); /* malloc'd within. */
+        hc[HDR__KEYWORDS].Value  = key_old_value;
+        hc[HDR__KEYWORDS].Length = key_old_length;
+        key_old_value = NULL;
       }
     }
   }
@@ -1967,7 +1991,11 @@ ARTpost(CHANNEL *cp)
   /* We have not parsed the Path: header yet.  We do not check for logipaddr
    * right now (it will be done afterwards and change data->Feedsite
    * in consequence).  We assign a feed site for the next call to ARTlog(). */
-  data->Feedsite = RChostname(cp);
+  if (cp->Address.ss_family != 0) {
+    data->Feedsite = RChostname(cp);
+  } else {
+    data->Feedsite = "localhost";
+  }
   if (data->Feedsite == NULL)
     data->Feedsite = CHANname(cp);
   data->FeedsiteLength = strlen(data->Feedsite);
@@ -2013,8 +2041,10 @@ ARTpost(CHANNEL *cp)
   }
   if (!artclean) {
     ARTlog(data, ART_REJECT, cp->Error);
+    /* If the article posting time has not been properly parsed, data->Posted
+     * will be negative or zero. */
     if (innconf->remembertrash && (Mode == OMrunning) &&
-	!InndHisRemember(HDR(HDR__MESSAGE_ID)))
+	!InndHisRemember(HDR(HDR__MESSAGE_ID), data->Posted))
       syslog(L_ERROR, "%s cant write history %s %m", LogName,
 	HDR(HDR__MESSAGE_ID));
     ARTreject(REJECT_OTHER, cp);
@@ -2047,7 +2077,7 @@ ARTpost(CHANNEL *cp)
                MaxLength(ME.Exclusions[j], ME.Exclusions[j]));
       ARTlog(data, ART_REJECT, cp->Error);
       if (innconf->remembertrash && (Mode == OMrunning) &&
-	  !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+	  !InndHisRemember(HDR(HDR__MESSAGE_ID), data->Posted))
 	syslog(L_ERROR, "%s cant write history %s %m", LogName,
 	  HDR(HDR__MESSAGE_ID));
       ARTreject(REJECT_SITE, cp);
@@ -2079,7 +2109,7 @@ ARTpost(CHANNEL *cp)
              cp->Error);
       ARTlog(data, ART_REJECT, cp->Error);
       if (innconf->remembertrash && (Mode == OMrunning) &&
-	  !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+	  !InndHisRemember(HDR(HDR__MESSAGE_ID), data->Posted))
 	syslog(L_ERROR, "%s cant write history %s %m", LogName,
 	  HDR(HDR__MESSAGE_ID));
       ARTreject(REJECT_FILTER, cp);
@@ -2110,7 +2140,7 @@ ARTpost(CHANNEL *cp)
              cp->Error);
       ARTlog(data, ART_REJECT, cp->Error);
       if (innconf->remembertrash && (Mode == OMrunning) &&
-	  !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+	  !InndHisRemember(HDR(HDR__MESSAGE_ID), data->Posted))
 	syslog(L_ERROR, "%s cant write history %s %m", LogName,
 	  HDR(HDR__MESSAGE_ID));
       ARTreject(REJECT_FILTER, cp);
@@ -2127,7 +2157,7 @@ ARTpost(CHANNEL *cp)
                MaxLength(HDR(HDR__DISTRIBUTION), HDR(HDR__DISTRIBUTION)));
       ARTlog(data, ART_REJECT, cp->Error);
       if (innconf->remembertrash && Mode == OMrunning &&
-	  !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+	  !InndHisRemember(HDR(HDR__MESSAGE_ID), data->Posted))
         syslog(L_ERROR, "%s cant write history %s %m", LogName,
 	  HDR(HDR__MESSAGE_ID));
       ARTreject(REJECT_DISTRIB, cp);
@@ -2145,7 +2175,7 @@ ARTpost(CHANNEL *cp)
                            data->Distribution.List[0]));
 	ARTlog(data, ART_REJECT, cp->Error);
         if (innconf->remembertrash && (Mode == OMrunning) &&
-	    !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+	    !InndHisRemember(HDR(HDR__MESSAGE_ID), data->Posted))
 	  syslog(L_ERROR, "%s cant write history %s %m",
 	    LogName, HDR(HDR__MESSAGE_ID));
 	ARTreject(REJECT_DISTRIB, cp);
@@ -2252,7 +2282,7 @@ ARTpost(CHANNEL *cp)
                    MaxLength(p, p));
           ARTlog(data, ART_REJECT, cp->Error);
           if (innconf->remembertrash && (Mode == OMrunning) &&
-              !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+              !InndHisRemember(HDR(HDR__MESSAGE_ID), data->Posted))
               syslog(L_ERROR, "%s cant write history %s %m",
                      LogName, HDR(HDR__MESSAGE_ID));
           ARTreject(REJECT_GROUP, cp);
@@ -2293,7 +2323,7 @@ ARTpost(CHANNEL *cp)
                MaxLength(ngp->Name, ngp->Name));
       ARTlog(data, ART_REJECT, cp->Error);
       if (innconf->remembertrash && (Mode == OMrunning) &&
-	  !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+	  !InndHisRemember(HDR(HDR__MESSAGE_ID), data->Posted))
 	syslog(L_ERROR, "%s cant write history %s %m", LogName,
 	  HDR(HDR__MESSAGE_ID));
       ARTreject(REJECT_UNAPP, cp);
@@ -2397,7 +2427,8 @@ ARTpost(CHANNEL *cp)
       ARTlog(data, ART_REJECT, cp->Error);
       if (!innconf->wanttrash) {
 	if (innconf->remembertrash && (Mode == OMrunning) &&
-	  !NoHistoryUpdate && !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+	  !NoHistoryUpdate && !InndHisRemember(HDR(HDR__MESSAGE_ID),
+                                               data->Posted))
 	  syslog(L_ERROR, "%s cant write history %s %m",
 	    LogName, HDR(HDR__MESSAGE_ID));
 	ARTreject(REJECT_GROUP, cp);
@@ -2410,7 +2441,8 @@ ARTpost(CHANNEL *cp)
          * which you explicitly excluded in your active file. */
   	if (!GroupMissing) {
 	  if (innconf->remembertrash && (Mode == OMrunning) &&
-	      !NoHistoryUpdate && !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+	      !NoHistoryUpdate && !InndHisRemember(HDR(HDR__MESSAGE_ID),
+                                                   data->Posted))
 	    syslog(L_ERROR, "%s cant write history %s %m",
 	      LogName, HDR(HDR__MESSAGE_ID));
 	  ARTreject(REJECT_GROUP, cp);
@@ -2447,7 +2479,7 @@ ARTpost(CHANNEL *cp)
       }
       ARTlog(data, ART_REJECT, cp->Error);
       if (innconf->remembertrash && (Mode == OMrunning) &&
-          !InndHisRemember(HDR(HDR__MESSAGE_ID)))
+          !InndHisRemember(HDR(HDR__MESSAGE_ID), data->Posted))
           syslog(L_ERROR, "%s cant write history %s %m",
                  LogName, HDR(HDR__MESSAGE_ID));
       ARTreject(REJECT_OTHER, cp);
