@@ -1,4 +1,4 @@
-/*  $Id: chan.c 7636 2007-08-06 17:31:19Z eagle $
+/*  $Id: chan.c 8557 2009-08-05 21:02:53Z iulius $
 **
 **  I/O channel (and buffer) processing.
 **
@@ -456,7 +456,8 @@ CHANname(CHANNEL *cp)
         snprintf(cp->Name, sizeof(cp->Name), "remconn:%d", cp->fd);
         break;
     case CTreject:
-        snprintf(cp->Name, sizeof(cp->Name), "%s rejected", RChostname(cp));
+        snprintf(cp->Name, sizeof(cp->Name), "%s rejected",
+                 cp->Address.ss_family == 0 ? "localhost" : RChostname(cp));
         break;
     case CTnntp:
         snprintf(cp->Name, sizeof(cp->Name), "%s:%d",
