@@ -1,4 +1,4 @@
-/*  $Id: innd.c 8160 2008-11-15 15:35:23Z iulius $
+/*  $Id: innd.c 8474 2009-05-17 13:12:15Z iulius $
 **
 **  Variable definitions, miscellany, and main().
 */
@@ -8,11 +8,12 @@
 
 #include "inn/innconf.h"
 #include "inn/messages.h"
+#include "inn/newsuser.h"
 #include "innperl.h"
 
 #define DEFINE_DATA
 #include "innd.h"
-#include "ov.h"
+#include "inn/ov.h"
 
 
 bool		Debug = false;
@@ -54,7 +55,7 @@ FILE *Errlog = NULL;
 #endif
 
 /* Internal prototypes. */
-static RETSIGTYPE       catch_terminate(int sig);
+static void             catch_terminate(int sig);
 static void             xmalloc_abort(const char *what, size_t size,
                                       const char *file, int line);
 
@@ -212,7 +213,7 @@ const ARTHEADER ARTheaders[] = {
 **  Signal handler to catch SIGTERM and similar signals and queue a clean
 **  shutdown.
 */
-static RETSIGTYPE
+static void
 catch_terminate(int sig)
 {
     GotTerminate = true;
@@ -249,8 +250,8 @@ CleanupAndExit(int status, const char *why)
     if (why)
         syslog(LOG_WARNING, "SERVER shutdown %s", why);
     else
-        syslog(LOG_WARNING, "SERVER shutdown received signal %d",
-               killer_signal);
+        syslog(LOG_WARNING, "SERVER shutdown received signal %lu",
+               (unsigned long) killer_signal);
     exit(status);
 }
 
@@ -274,10 +275,6 @@ JustCleanup(void)
     NGclose();
     SMshutdown();
 
-#if DO_TCL
-    TCLclose();
-#endif
-
 #if DO_PERL
     PerlFilter(false);
     PerlClose();
@@ -312,7 +309,7 @@ ReopenLog(FILE *F)
 	return;
 
     path = concatpath(innconf->pathlog,
-                      (F == stdout) ? _PATH_LOGFILE : _PATH_ERRLOG);
+                      (F == stdout) ? INN_PATH_LOGFILE : INN_PATH_ERRLOG);
     oldpath = concat(path, ".old", (char *) 0);
     if (rename(path, oldpath) < 0)
         syswarn("SERVER cant rename %s to %s", path, oldpath);
@@ -343,11 +340,10 @@ main(int ac, char *av[])
 {
     const char *name, *p;
     char *path;
-    char *t;
     bool flag;
     static char		WHEN[] = "PID file";
-    int			i, j, fd[MAX_SOCKETS + 1];
-    char		buff[SMBUF], *path1, *path2;
+    int			i;
+    char		buff[SMBUF];
     FILE		*F;
     bool		ShouldFork;
     bool		ShouldRenumber;
@@ -357,6 +353,9 @@ main(int ac, char *av[])
 #if	defined(_DEBUG_MALLOC_INC)
     union malloptarg	m;
 #endif	/* defined(_DEBUG_MALLOC_INC) */
+#if DO_PERL
+    char                *path1, *path2;
+#endif
 
     /* Set up the pathname, first thing, and teach our error handlers about
        the name of the program. */
@@ -374,10 +373,9 @@ main(int ac, char *av[])
     message_handlers_warn(2, message_log_stderr, message_log_syslog_err);
     message_handlers_notice(1, message_log_syslog_notice);
 
-    /* Make sure innd is not running as root.  innd must be either started
-       via inndstart or use a non-privileged port. */
-    if (getuid() == 0 || geteuid() == 0)
-        die("SERVER must be run as user news, not root (use inndstart)");
+    /* Make sure innd is running as the correct user.  If it is started as
+       root, switch to running as the news user. */
+    ensure_news_user_grp(true, true);
 
     /* Handle malloc debugging. */
 #if	defined(_DEBUG_MALLOC_INC)
@@ -397,7 +395,6 @@ main(int ac, char *av[])
     ShouldFork = true;
     ShouldRenumber = false;
     ShouldSyntaxCheck = false;
-    fd[0] = fd[1] = -1;
 
     /* Set some options from inn.conf that can be overridden with
        command-line options if they exist, so read inn.conf first. */
@@ -406,11 +403,21 @@ main(int ac, char *av[])
 
     /* Parse JCL. */
     CCcopyargv(av);
-    while ((i = getopt(ac, av, "ac:Cdfi:l:m:o:Nn:p:P:rst:uH:T:X:")) != EOF)
+    while ((i = getopt(ac, av, "4:6:ac:CdfH:i:l:m:n:No:P:rst:T:uX:")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
 	    /* NOTREACHED */
+        case '4':
+            if (innconf->bindaddress)
+                free(innconf->bindaddress);
+            innconf->bindaddress = xstrdup(optarg);
+            break;
+        case '6':
+            if (innconf->bindaddress6)
+                free(innconf->bindaddress6);
+            innconf->bindaddress6 = xstrdup(optarg);
+            break;
 	case 'a':
 	    AnyIncoming = true;
 	    break;
@@ -432,10 +439,6 @@ main(int ac, char *av[])
 	case 'i':
 	    innconf->maxconnections = atoi(optarg);
 	    break;
-	case 'I':
-	    if (innconf->bindaddress) free(innconf->bindaddress);
-	    innconf->bindaddress = xstrdup(optarg);
-	    break;
 	case 'l':
 	    innconf->maxartsize = atol(optarg);
 	    break;
@@ -469,22 +472,6 @@ main(int ac, char *av[])
 	case 'o':
 	    MaxOutgoing = atoi(optarg);
 	    break;
-	case 'p':
-	    /* Silently ignore multiple -p flags, in case ctlinnd xexec
-	       called inndstart. */
-	    if (fd[0] != -1)
-		break;
-	    t = xstrdup(optarg);
-	    p = strtok(t, ",");
-	    j = 0;
-	    do {
-		fd[j++] = atoi(p);
-		if (j == MAX_SOCKETS)
-		    break;
-	    } while ((p = strtok(NULL, ",")) != NULL);
-	    fd[j] = -1;
-	    free(t);
-	    break;
 	case 'P':
 	    innconf->port = atoi(optarg);
 	    break;
@@ -578,12 +565,12 @@ main(int ac, char *av[])
            articles and stderr is used to log serious error conditions (as
            well as to capture stderr from embedded filters).  Both are
            normally fully buffered. */
-        path = concatpath(innconf->pathlog, _PATH_LOGFILE);
+        path = concatpath(innconf->pathlog, INN_PATH_LOGFILE);
         if (freopen(path, "a", stdout) == NULL)
             sysdie("SERVER cant freopen stdout to %s", path);
         setvbuf(stdout, NULL, _IOFBF, LOG_BUFSIZ);
         free(path);
-        path = concatpath(innconf->pathlog, _PATH_ERRLOG);
+        path = concatpath(innconf->pathlog, INN_PATH_ERRLOG);
         if (freopen(path, "a", stderr) == NULL)
             sysdie("SERVER cant freopen stderr to %s", path);
         setvbuf(stderr, NULL, _IOLBF, BUFSIZ);
@@ -596,17 +583,15 @@ main(int ac, char *av[])
     if (innconf->enableoverview && !OVopen(OV_WRITE))
         die("SERVER cant open overview method");
 
-    /* Always attempt to increase the number of open file descriptors.  If
-       we're not root, this may just fail quietly. */
+    /* Attempt to increase the number of open file descriptors. */
     if (innconf->rlimitnofile > 0)
-        setfdlimit(innconf->rlimitnofile);
+        if (setfdlimit(innconf->rlimitnofile) < 0)
+            syswarn("SERVER cant set file descriptor limit");
 
     /* Get number of open channels. */
     i = getfdlimit();
-    if (i < 0) {
-	syslog(L_FATAL, "%s cant get file descriptor limit: %m", LogName);
-	exit(1);
-    }
+    if (i < 0)
+        sysdie("SERVER cant get file descriptor limit");
 
     /* There is no file descriptor limit on some hosts; for those, cap at
        MaxOutgoing plus maxconnections plus 20, or 5000, whichever is larger. 
@@ -629,7 +614,7 @@ main(int ac, char *av[])
 
     /* See if another instance is alive. */
     if (PID == NULL)
-	PID = concatpath(innconf->pathrun, _PATH_SERVERPID);
+	PID = concatpath(innconf->pathrun, INN_PATH_SERVERPID);
     if ((F = fopen(PID, "r")) != NULL) {
 	if (fgets(buff, sizeof buff, F) != NULL
 	 && ((pid = (pid_t) atol(buff)) > 0)
@@ -641,26 +626,26 @@ main(int ac, char *av[])
 	fclose(F);
     }
 
-    if (GetTimeInfo(&Now) < 0)
-	syslog(L_ERROR, "%s cant gettimeinfo %m", LogName);
+    if (gettimeofday(&Now, NULL) < 0)
+	syslog(L_ERROR, "%s cant gettimeofday %m", LogName);
 
     /* Set up signal and error handlers. */
     xmalloc_error_handler = xmalloc_abort;
     xsignal(SIGHUP, catch_terminate);
     xsignal(SIGTERM, catch_terminate);
 
-    /* Set up the various parts of the system.  Channel feeds start
-       processes so call PROCsetup before ICDsetup.  NNTP needs to know if
-       it's a slave, so call RCsetup before NCsetup. */
+    /* Set up the various parts of the system.  Channel feeds start processes
+       so call PROCsetup before ICDsetup.  NNTP needs to know if it's a slave,
+       so call RCsetup before NCsetup.  RCsetup calls innbind and waits for
+       it, so call PROCsetup after RCsetup to not interpose a signal
+       handler. */
     CHANsetup(i);
-    PROCsetup(10);
     if (Mode == OMrunning)
         InndHisOpen();
     CCsetup();
     LCsetup();
-    RCsetup(fd[0]);
-    for (i = 1; fd[i] != -1; i++)
-	RCsetup(fd[i]);
+    RCsetup();
+    PROCsetup(10);
     WIPsetup();
     NCsetup();
     ARTsetup();
@@ -706,16 +691,10 @@ main(int ac, char *av[])
 	}
     }
 
-#if DO_TCL
-    TCLsetup();
-    if (!filter)
-	TCLfilter(false);
-#endif /* DO_TCL */
-
 #if DO_PERL
     /* Load the Perl code */
-    path1 = concatpath(innconf->pathfilter, _PATH_PERL_STARTUP_INND);
-    path2 = concatpath(innconf->pathfilter, _PATH_PERL_FILTER_INND);
+    path1 = concatpath(innconf->pathfilter, INN_PATH_PERL_STARTUP_INND);
+    path2 = concatpath(innconf->pathfilter, INN_PATH_PERL_FILTER_INND);
     PERLsetup(path1, path2, "filter_art");
     free(path1);
     free(path2);
