@@ -1,4 +1,4 @@
-/*  $Id: perl.c 7951 2008-08-05 19:41:17Z iulius $
+/*  $Id: perl.c 8344 2009-02-22 21:02:17Z iulius $
 **
 **  Perl filtering support for innd.
 **
@@ -14,18 +14,10 @@
 **
 **  Future work:
 **
-**   - What we're doing with Path headers right now doesn't work for folded
-**     headers.  It's also kind of gross.  There has to be a better way of
-**     handling this.
-**
 **   - The breakdown between this file, lib/perl.c, and nnrpd/perl.c should
 **     be rethought, ideally in the light of supporting multiple filters in
 **     different languages.
 **
-**   - We're still calling strlen() on artBody, which should be avoidable
-**     since we've already walked it several times.  We should just cache
-**     the length somewhere for speed.
-**
 **   - Variable and key names should be standardized between this and nnrpd.
 **
 **   - The XS code is still calling CC* functions.  The common code between
@@ -43,6 +35,7 @@
 #if DO_PERL
 
 #include "clibrary.h"
+#include "inn/wire.h"
 #include "innd.h"
 
 #include <EXTERN.h>
@@ -52,8 +45,15 @@
 
 #include "innperl.h"
 
-/* From art.c.  Ew.  Need header parsing that doesn't use globals. */
-extern char             *filterPath;
+/* Prototypes for XS functions. */
+XS(XS_INN_addhist);
+XS(XS_INN_article);
+XS(XS_INN_cancel);
+XS(XS_INN_filesfor);
+XS(XS_INN_havehist);
+XS(XS_INN_head);
+XS(XS_INN_newsgroup);
+
 
 /*
 **  Run an incoming article through the Perl article filter.  Returns NULL
@@ -75,33 +75,32 @@ PLartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
     filter = perl_get_cv("filter_art", 0);
     if (!filter) return NULL;
 
-    /* Create %hdr and stash a copy of every known header.  Path has to be
-       handled separately since it's been munged by article processing. */
+    /* Create %hdr and stash a copy of every known header. */
     hdr = perl_get_hv("hdr", 1);
     for (i = 0 ; i < MAX_ARTHEADER ; i++) {
 	if (HDR_FOUND(i)) {
 	    hp = &ARTheaders[i];
-            hv_store(hdr, (char *) hp->Name, hp->Size, newSVpv(HDR(i), 0), 0);
+            (void) hv_store(hdr, (char *) hp->Name, hp->Size, newSVpv(HDR(i), 0), 0);
 	}
     }
 
     /* Store the article body.  We don't want to make another copy of it,
-     * since it could potentially be quite large.  In testing, this produced
-     * a 17% speed improvement over making a copy of the article body
-     * for a fairly heavy filter.
-     * Available since Perl 5.7.1, newSVpvn_share allows to avoid such
-     * a copy (getting round its use for older versions of Perl leads
-     * to unreliable SV * bodies as for regexps).  And for Perl not to
-     * compute a hash for artBody, we give it "42". */
+       since it could potentially be quite large.  In testing, this produced
+       a 17% speed improvement over making a copy of the article body
+       for a fairly heavy filter.
+       Available since Perl 5.7.1, newSVpvn_share allows to avoid such
+       a copy (getting round its use for older versions of Perl leads
+       to unreliable SV * bodies as for regexps).  And for Perl not to
+       compute a hash for artBody, we give it "42". */
     if (artBody) {
 #if (PERL_REVISION == 5) && ((PERL_VERSION < 7) || ((PERL_VERSION == 7) && (PERL_SUBVERSION < 1)))
-        hv_store(hdr, "__BODY__", 8, newSVpv(artBody, artLen), 0);
+        (void) hv_store(hdr, "__BODY__", 8, newSVpv(artBody, artLen), 0);
 #else
-        hv_store(hdr, "__BODY__", 8, newSVpvn_share(artBody, artLen, 42), 0);
+        (void) hv_store(hdr, "__BODY__", 8, newSVpvn_share(artBody, artLen, 42), 0);
 #endif /* Perl < 5.7.1 */
     }
 
-    hv_store(hdr, "__LINES__", 9, newSViv(lines), 0);
+    (void) hv_store(hdr, "__LINES__", 9, newSViv(lines), 0);
 
     ENTER;
     SAVETMPS;
@@ -197,20 +196,24 @@ PLmode(OPERATINGMODE Mode, OPERATINGMODE NewMode, char *reason)
     mode = perl_get_hv("mode", 1);
 
     if (Mode == OMrunning)
-        hv_store(mode, "Mode", 4, newSVpv("running", 0), 0);
+        (void) hv_store(mode, "Mode", 4, newSVpv("running", 7), 0);
     if (Mode == OMpaused)
-        hv_store(mode, "Mode", 4, newSVpv("paused", 0), 0);
+        (void) hv_store(mode, "Mode", 4, newSVpv("paused", 6), 0);
     if (Mode == OMthrottled)
-        hv_store(mode, "Mode", 4, newSVpv("throttled", 0), 0);
+        (void) hv_store(mode, "Mode", 4, newSVpv("throttled", 9), 0);
+    if (Mode == OMshutdown)
+        (void) hv_store(mode, "Mode", 4, newSVpv("shutdown", 8), 0);
 
     if (NewMode == OMrunning)
-        hv_store(mode, "NewMode", 7, newSVpv("running", 0), 0);
+        (void) hv_store(mode, "NewMode", 7, newSVpv("running", 7), 0);
     if (NewMode == OMpaused)
-        hv_store(mode, "NewMode", 7, newSVpv("paused", 0), 0);
+        (void) hv_store(mode, "NewMode", 7, newSVpv("paused", 6), 0);
     if (NewMode == OMthrottled)
-        hv_store(mode, "NewMode", 7, newSVpv("throttled", 0), 0);
+        (void) hv_store(mode, "NewMode", 7, newSVpv("throttled", 9), 0);
+    if (NewMode == OMshutdown)
+        (void) hv_store(mode, "NewMode", 7, newSVpv("shutdown", 8), 0);
 
-    hv_store(mode, "reason", 6, newSVpv(reason, 0), 0);
+    (void) hv_store(mode, "reason", 6, newSVpv(reason, 0), 0);
 
     PUSHMARK(SP);
     perl_call_sv((SV *) filter, G_EVAL|G_DISCARD|G_NOARGS);
@@ -281,6 +284,9 @@ XS(XS_INN_addhist)
     char        tbuff[32];
     char*       parambuf[6];
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items < 1 || items > 5)
         croak("Usage INN::addhist(msgid,[arrival,articletime,expire,token])");
 
@@ -296,7 +302,7 @@ XS(XS_INN_addhist)
 
     /* The token defaults to an empty string. */
     if (i == 4)
-        parambuf[4] = "";
+        parambuf[4] = (char *) "";
 
     parambuf[5] = NULL;
 
@@ -323,6 +329,9 @@ XS(XS_INN_article)
     char *      p;
     size_t      len;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
 	croak("Usage: INN::article(msgid)");
 
@@ -333,7 +342,7 @@ XS(XS_INN_article)
     /* Retrieve the article and convert it from wire format. */
     art = SMretrieve(token, RETR_ALL);
     if (art == NULL) XSRETURN_UNDEF;
-    p = FromWireFmt(art->data, art->len, &len);
+    p = wire_to_native(art->data, art->len, &len);
     SMfreearticle(art);
 
     /* Push a copy of the article onto the Perl stack, free our temporary
@@ -354,6 +363,9 @@ XS(XS_INN_cancel)
     char        *msgid;
     char        *parambuf[2];
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::cancel(msgid)");
 
@@ -380,6 +392,9 @@ XS(XS_INN_filesfor)
     char        *msgid;
     TOKEN       token;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::filesfor(msgid)");
 
@@ -400,6 +415,9 @@ XS(XS_INN_havehist)
     dXSARGS;
     char        *msgid;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::havehist(msgid)");
 
@@ -425,6 +443,9 @@ XS(XS_INN_head)
     char *      p;
     size_t      len;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::head(msgid)");
 
@@ -435,7 +456,7 @@ XS(XS_INN_head)
     /* Retrieve the article header and convert it from wire format. */
     art = SMretrieve(token, RETR_HEAD);
     if (art == NULL) XSRETURN_UNDEF;
-    p = FromWireFmt(art->data, art->len, &len);
+    p = wire_to_native(art->data, art->len, &len);
     SMfreearticle(art);
 
     /* Push a copy of the article header onto the Perl stack, free our
@@ -458,6 +479,9 @@ XS(XS_INN_newsgroup)
     char *      end;
     int         size;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::newsgroup(group)");
     newsgroup = (char *) SvPV(ST(0), PL_na);
@@ -485,13 +509,13 @@ XS(XS_INN_newsgroup)
 void
 PLxsinit(void)
 {
-    newXS("INN::addhist", XS_INN_addhist, "perl.c");
-    newXS("INN::article", XS_INN_article, "perl.c");
-    newXS("INN::cancel", XS_INN_cancel, "perl.c");
-    newXS("INN::havehist", XS_INN_havehist, "perl.c");
-    newXS("INN::head", XS_INN_head, "perl.c");
-    newXS("INN::newsgroup", XS_INN_newsgroup, "perl.c");
-    newXS("INN::filesfor", XS_INN_filesfor, "perl.c");
+    inn_newXS("INN::addhist", XS_INN_addhist, "perl.c");
+    inn_newXS("INN::article", XS_INN_article, "perl.c");
+    inn_newXS("INN::cancel", XS_INN_cancel, "perl.c");
+    inn_newXS("INN::havehist", XS_INN_havehist, "perl.c");
+    inn_newXS("INN::head", XS_INN_head, "perl.c");
+    inn_newXS("INN::newsgroup", XS_INN_newsgroup, "perl.c");
+    inn_newXS("INN::filesfor", XS_INN_filesfor, "perl.c");
 }
 
 #endif /* defined(DO_PERL) */
