@@ -1,4 +1,4 @@
-/*  $Id: misc.h,v 1.15.2.1 2004/01/25 20:07:10 rra Exp $
+/*  $Id: misc.h 8243 2008-12-21 11:50:47Z iulius $
 **
 **  Miscellaneous utility functions for innfeed.
 **
@@ -42,6 +42,9 @@ typedef void (*EndpWorkCbk) (EndPoint ep, void *data) ;
 /* debugging information */
 extern unsigned int loggingLevel ;     /* if 0 then d_printf is a no-op */
 
+/* used by timeToString with strftime(3) */
+extern char *timeToStringFormat ;
+
 /* the current count of file desccriptors */
 extern unsigned int openfds ;
 
@@ -114,12 +117,12 @@ bool isOlder (const char *file1, const char *file2) ;
 /* converts val into a printable string */
 const char *boolToString (bool val) ;
 
+/* strftime with "%a %b %d %H:%M:%S %Y" (like ctime without linefeed) */
+char* timeToString (time_t time, char* buffer, size_t size) ;
+
 /* memory leak checker helper. */
 void addPointerFreedOnExit (char *pointerToFree) ;
 
-/* splice direcotory and fname together and return free'able string */
-char *buildFilename (const char *directory, const char *fname) ;
-
 /* string the file opened by FP to the give SIZE. The NEWNAME is the name
    of the file to have after truncation. FP will be reopened for writing on
    the new file and will be positioned at the end */
