@@ -1,32 +1,30 @@
-/*  $Id: clientlib.c,v 1.8.2.1 2000/06/21 23:31:12 rra Exp $
+/*  $Id: clientlib.c 8469 2009-05-16 12:43:53Z iulius $
 **
 **  Routines compatible with the NNTP "clientlib" routines.
 */
 #include "config.h"
 #include "clibrary.h"
 
-#include "libinn.h"
+#include "inn/innconf.h"
+#include "inn/libinn.h"
 #include "nntp.h"
 
 
 FILE	*ser_rd_fp = NULL;
 FILE	*ser_wr_fp = NULL;
-char	ser_line[NNTP_STRLEN + 2];
-void put_server(char *buff);
-int get_server(char *buff, int buffsize);
+char	ser_line[NNTP_MAXLEN_COMMAND + 2];
 
 
 /*
 **  Get the name of the NNTP server.  Ignore the filename; we use
 **  our own configuration stuff.  Return pointer to static data.
 */
-/* ARGSUSED0 */
-char *getserverbyfile(char *file)
+char *
+getserverbyfile(char *file UNUSED)
 {
     static char	buff[256];
 
-    strncpy(buff, innconf->server, sizeof(buff));
-    buff[255] = '\0';
+    strlcpy(buff, innconf->server, sizeof(buff));
     return buff;
 }
 
@@ -35,16 +33,19 @@ char *getserverbyfile(char *file)
 **  Get a connection to the remote news server.  Return server's reply
 **  code or -1 on error.
 */
-int server_init(char *host, int port)
+int
+server_init(char *host, int port)
 {
-    char	line2[NNTP_STRLEN];
+    char	line2[NNTP_MAXLEN_COMMAND];
 
     /* This interface may be used by clients that assume C News behavior and
        won't read inn.conf themselves. */
     if (innconf == NULL)
-        if (ReadInnConf() < 0) return -1;
+        if (!innconf_read(NULL))
+            return -1;
 
-    if (NNTPconnect(host, port, &ser_rd_fp, &ser_wr_fp, ser_line) < 0) {
+    if (NNTPconnect(host, port, &ser_rd_fp, &ser_wr_fp, ser_line,
+                    sizeof(ser_line)) < 0) {
 	if (ser_line[0] == '\0')
 	    /* I/O problem. */
 	    return -1;
@@ -57,8 +58,8 @@ int server_init(char *host, int port)
     put_server("mode reader");
     if (get_server(line2, (int)sizeof line2) < 0)
 	return -1;
-    if (atoi(line2) != NNTP_BAD_COMMAND_VAL)
-	(void)strcpy(ser_line, line2);
+    if (atoi(line2) != NNTP_ERR_COMMAND)
+	strlcpy(ser_line, line2, sizeof(ser_line));
 
     /* Connected; return server's reply code. */
     return atoi(ser_line);
@@ -73,34 +74,35 @@ int server_init(char *host, int port)
 **  Print a message based on the the server's initial response.
 **  Return -1 if server wants us to go away.
 */
-int handle_server_response(int response, char *host)
+int
+handle_server_response(int response, char *host)
 {
     char	*p;
 
     switch (response) {
     default:
-	(void)printf("Unknown response code %d from %s.\n", response, host);
+	printf("Unknown response code %d from %s.\n", response, host);
 	return -1;
-     case NNTP_GOODBYE_VAL:
+     case NNTP_FAIL_TERMINATING:
 	if (atoi(ser_line) == response) {
 	    p = &ser_line[strlen(ser_line) - 1];
 	    if (*p == '\n' && *--p == '\r')
 		*p = '\0';
 	    if (p > &ser_line[3]) {
-		(void)printf("News server %s unavailable: %s\n", host,
+		printf("News server %s unavailable: %s\n", host,
 			&ser_line[4]);
 		return -1;
 	    }
 	}
-	(void)printf("News server %s unavailable, try later.\n", host);
+	printf("News server %s unavailable, try later.\n", host);
 	return -1;
-    case NNTP_ACCESS_VAL:
-	(void)printf(CANTUSE, host);
+    case NNTP_ERR_ACCESS:
+	printf(CANTUSE, host);
 	return -1;
-    case NNTP_NOPOSTOK_VAL:
-	(void)printf("%s.\n", CANTPOST);
+    case NNTP_OK_BANNER_NOPOST:
+	printf("%s.\n", CANTPOST);
 	/* FALLTHROUGH */
-    case NNTP_POSTOK_VAL:
+    case NNTP_OK_BANNER_POST:
 	break;
     }
     return 0;
@@ -110,10 +112,11 @@ int handle_server_response(int response, char *host)
 /*
 **  Send a line of text to the server.
 */
-void put_server(char *buff)
+void
+put_server(const char *buff)
 {
-    (void)fprintf(ser_wr_fp, "%s\r\n", buff);
-    (void)fflush(ser_wr_fp);
+    fprintf(ser_wr_fp, "%s\r\n", buff);
+    fflush(ser_wr_fp);
 }
 
 
@@ -121,9 +124,10 @@ void put_server(char *buff)
 **  Get a line of text from the server, strip trailing \r\n.
 **  Return -1 on error.
 */
-int get_server(char *buff, int buffsize)
+int
+get_server(char *buff, int buffsize)
 {
-    char	        *p;
+    char	*p;
 
     if (fgets(buff, buffsize, ser_rd_fp) == NULL)
 	return -1;
@@ -138,17 +142,17 @@ int get_server(char *buff, int buffsize)
 **  Send QUIT and close the server.
 */
 void
-close_server()
+close_server(void)
 {
-    char	buff[NNTP_STRLEN];
+    char	buff[NNTP_MAXLEN_COMMAND];
 
     if (ser_wr_fp != NULL && ser_rd_fp != NULL) {
 	put_server("QUIT");
-	(void)fclose(ser_wr_fp);
+	fclose(ser_wr_fp);
 	ser_wr_fp = NULL;
 
-	(void)get_server(buff, (int)sizeof buff);
-	(void)fclose(ser_rd_fp);
+	get_server(buff, (int)sizeof buff);
+	fclose(ser_rd_fp);
 	ser_rd_fp = NULL;
     }
 }
