@@ -1,4 +1,4 @@
-/*  $Id: commands.c 9019 2010-03-19 21:27:15Z iulius $
+/*  $Id: commands.c 8255 2008-12-24 22:27:07Z iulius $
 **
 **  Routines for NNTP commands:  manipulation and checks.
 */
@@ -27,11 +27,11 @@ IsValidKeyword(const char *string)
         return false;
 
     /* Begins with a letter. */
-    if (!isalpha((unsigned char) string[0]))
+    if (!CTYPE(isalpha, string[0]))
         return false;
 
     for (; *string != '\0'; string++) {
-        if (isalnum((unsigned char) *string) || *string == '.' || *string == '-')
+        if (CTYPE(isalnum, *string) || *string == '.' || *string == '-')
             len++;
         else
             return false;
