@@ -1,4 +1,4 @@
-/*  $Id: dbz.c 7775 2008-04-16 16:52:48Z iulius $
+/*  $Id: dbz.c 8556 2009-08-05 21:01:48Z iulius $
 **
 **  dbz database implementation V6.1.1
 **
@@ -77,7 +77,7 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 
-#include "dbz.h"
+#include "inn/dbz.h"
 #include "inn/messages.h"
 #include "inn/innconf.h"
 #include "inn/mmap.h"
