@@ -1,4 +1,4 @@
-/*  $Id: numbers.c 9019 2010-03-19 21:27:15Z iulius $
+/*  $Id: numbers.c 8009 2008-09-06 16:43:51Z iulius $
 **
 **  Routines for numbers:  manipulation and checks.
 */
@@ -28,7 +28,7 @@ IsValidArticleNumber(const char *string)
    
     for (; *p != '\0'; p++) {
         len++;
-        if (!isdigit((unsigned char) *p))
+        if (!CTYPE(isdigit, *p))
             return false;
     }
 
