@@ -1,4 +1,4 @@
-/*  $Id: snprintf.c 7651 2007-08-20 10:28:34Z iulius $
+/*  $Id: snprintf.c 8630 2009-09-09 20:06:53Z iulius $
 **
 **  Replacement for a missing snprintf or vsnprintf.
 **
@@ -113,7 +113,7 @@
 #define LDOUBLE double
 #endif
 
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_LONG_INT
 # define LLONG long long
 #else
 # define LLONG long
