@@ -1,4 +1,4 @@
-/*  $Id: uwildmat.c 8378 2009-03-09 19:45:13Z iulius $
+/*  $Id: uwildmat.c 8637 2009-09-28 19:59:01Z iulius $
 **
 **  wildmat pattern matching with Unicode UTF-8 extensions.
 **
@@ -93,13 +93,54 @@ utf8_length(const unsigned char *start, const unsigned char *end)
     if (end != NULL && (end - start + 1) < length)
         return 1;
     left = length - 1;
-    p = start + 1;
     for (p = start + 1; left > 0 && (*p & 0xc0) == 0x80; p++)
         left--;
     return (left == 0) ? length : 1;
 }
 
 
+/*
+**  Check whether a string contains only valid UTF-8 characters.
+*/
+bool
+is_valid_utf8(const char *text)
+{
+    unsigned char mask;
+    const unsigned char *p;
+    int length;
+    int left;
+
+    for (p = (const unsigned char *)text; *p != '\0';) {
+        mask = 0x80;
+        length = 0;
+
+        /* Find out the expected length of the character. */
+        for (; mask > 0 && (*p & mask) == mask; mask >>= 1)
+            length++;
+
+        p++;
+
+        /* Valid ASCII. */
+        if (length == 0)
+            continue;
+        
+        /* Invalid length. */
+        if (length < 2 || length > 6)
+            return false;
+
+        /* Check that each byte looks like 10xxxxxx, except for the first. */
+        left = length - 1;
+        for (; left > 0 && (*p & 0xc0) == 0x80; p++)
+            left--;
+
+        if (left > 0)
+            return false;
+    }
+
+    return true;
+}
+
+
 /*
 **  Convert a UTF-8 character to UCS-4.  Takes a pointer to the start of the
 **  character and to the last octet of the string, and to a uint32_t into
@@ -235,7 +276,7 @@ match_pattern(const unsigned char *text, const unsigned char *start,
 
                Optimization: If the character after the * in the pattern
                isn't a metacharacter (the common case), then the * has to
-               consume characters at least up to the next occurance of that
+               consume characters at least up to the next occurrence of that
                character in the text.  Scan forward for those points rather
                than recursing at every possible point to save the extra
                function call overhead. */
@@ -286,7 +327,7 @@ match_pattern(const unsigned char *text, const unsigned char *start,
 
 /*
 **  Takes text and a wildmat expression; a wildmat expression is a
-**  comma-separated list of wildmat patterns, optionally preceeded by ! to
+**  comma-separated list of wildmat patterns, optionally preceded by ! to
 **  invert the sense of the expression.  Returns WILDMAT_MATCH if that
 **  expression matches the text, WILDMAT_FAIL otherwise.  If allowpoison is
 **  set, allow @ to introduce a poison expression (the same as !, but if it
