@@ -1,4 +1,4 @@
-/*  $Id: article.c 8209 2008-12-06 08:34:35Z iulius $
+/*  $Id: article.c 8519 2009-06-20 12:20:03Z iulius $
 **
 **  Article-related routines.
 */
@@ -992,19 +992,33 @@ CMDover(int ac, char *av[])
             HasNotReplied = false;
         }
 
-	vector = overview_split(data, len, NULL, vector);
-	r = overview_getheader(vector, OVERVIEW_MESSAGE_ID, OVextra);
+        vector = overview_split(data, len, NULL, vector);
+        r = overview_get_standard_header(vector, OVERVIEW_MESSAGE_ID);
+        if (r == NULL) {
+            if (PERMaccessconf->nnrpdoverstats) {
+                gettimeofday(&stv, NULL);
+            }
+            continue;
+        }
 	cache_add(HashMessageID(r), token);
 	free(r);
 	if (VirtualPathlen > 0 && overhdr_xref != -1) {
-	    if ((size_t)(overhdr_xref + 1) >= vector->count)
-		continue;
+            if ((size_t)(overhdr_xref + 1) >= vector->count) {
+                if (PERMaccessconf->nnrpdoverstats) {
+                    gettimeofday(&stv, NULL);
+                }
+                continue;
+            }
 	    p = vector->strings[overhdr_xref] + sizeof("Xref: ") - 1;
 	    while ((p < data + len) && *p == ' ')
 		++p;
 	    q = memchr(p, ' ', data + len - p);
-	    if (q == NULL)
-		continue;
+            if (q == NULL) {
+                if (PERMaccessconf->nnrpdoverstats) {
+                    gettimeofday(&stv, NULL);
+                }
+                continue;
+            }
 	    if(useIOb) {
 		SendIOb(data, p - data);
 		SendIOb(VirtualPath, VirtualPathlen - 1);
@@ -1078,7 +1092,7 @@ CMDpat(int ac, char *av[])
     bool		IsBytes, IsLines;
     bool                IsMetaBytes, IsMetaLines;
     bool		DidReply, HasNotReplied;
-    char		*header;
+    const char		*header;
     char		*pattern;
     char		*text;
     int			Overview;
@@ -1119,9 +1133,9 @@ CMDpat(int ac, char *av[])
     /* Make these changes because our overview database does
      * not currently know metadata names. */
     if (IsMetaBytes)
-        header = xstrdup("Bytes");
+        header = "Bytes";
     if (IsMetaLines)
-        header = xstrdup("Lines");
+        header = "Lines";
 
     /* We only allow :bytes and :lines for metadata. */
     if (!IsMetaLines && !IsMetaBytes) {
@@ -1279,7 +1293,11 @@ CMDpat(int ac, char *av[])
                 HasNotReplied = false;
             }
 	    vector = overview_split(data, len, NULL, vector);
-	    p = overview_getheader(vector, Overview, OVextra);
+            if (Overview < OVERVIEW_MAX) {
+                p = overview_get_standard_header(vector, Overview);
+            } else {
+                p = overview_get_extra_header(vector, header);
+            }
 	    if (p != NULL) {
 		if (PERMaccessconf->virtualhost &&
 			   Overview == overhdr_xref) {
