@@ -1,66 +1,66 @@
-/*  $Id: list.c 7953 2008-08-23 13:45:02Z iulius $
+/*  $Id: list.c 8409 2009-04-11 22:13:41Z iulius $
 **
-**  List commands.
+**  LIST commands.
 */
 
 #include "config.h"
 #include "clibrary.h"
 
 #include "nnrpd.h"
-#include "ov.h"
+#include "inn/ov.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
 
 typedef struct _LISTINFO {
     const char *method;
     const char * File;
-    void (*impl)(struct _LISTINFO *);
+    void (*impl)(struct _LISTINFO *, int ac, char *av[]);
     bool         Required;
     const char * Items;
     const char * Format;
 } LISTINFO;
 
-static void cmd_list_schema(LISTINFO *lp);
-static void cmd_list_extensions(LISTINFO *lp);
+static void cmd_list_schema(LISTINFO *lp, int ac, char *av[]);
+static void cmd_list_headers(LISTINFO *lp, int ac, char *av[]);
 
 static LISTINFO		INFOactive = {
-    "active", _PATH_ACTIVE, NULL, true, "active newsgroups",
+    "ACTIVE", INN_PATH_ACTIVE, NULL, true, "active newsgroups",
     "Newsgroups in form \"group high low flags\""
 };
 static LISTINFO		INFOactivetimes = {
-    "active.times", _PATH_ACTIVETIMES, NULL, false, "creation times",
+    "ACTIVE.TIMES", INN_PATH_ACTIVETIMES, NULL, false, "creation times",
     "Group creations in form \"name time who\""
 };
 static LISTINFO		INFOdistribs = {
-    "distributions", _PATH_NNRPDIST, NULL, false, "newsgroup distributions",
+    "DISTRIBUTIONS", INN_PATH_NNRPDIST, NULL, false, "newsgroup distributions",
     "Distributions in form \"area description\""
 };
+static LISTINFO         INFOheaders = {
+    "HEADERS", NULL, cmd_list_headers, true, "supported headers and metadata",
+    "Headers and metadata items supported"
+};
 static LISTINFO               INFOsubs = {
-    "subscriptions", _PATH_NNRPSUBS, NULL, false, "automatic group subscriptions",
-    "Subscriptions in form \"group\""
+    "SUBSCRIPTIONS", INN_PATH_NNRPSUBS, NULL, false,
+    "automatic group subscriptions", "Subscriptions in form \"group\""
 };
 static LISTINFO		INFOdistribpats = {
-    "distrib.pats", _PATH_DISTPATS, NULL, false, "distribution patterns",
+    "DISTRIB.PATS", INN_PATH_DISTPATS, NULL, false, "distribution patterns",
     "Default distributions in form \"weight:pattern:value\""
 };
-static LISTINFO		INFOextensions = {
-    "extensions", NULL, cmd_list_extensions, false, "supported extensions",
-    "Supported NNTP extensions"
-};
 static LISTINFO		INFOgroups = {
-    "newsgroups", _PATH_NEWSGROUPS, NULL, false, "newsgroup descriptions",
+    "NEWSGROUPS", INN_PATH_NEWSGROUPS, NULL, true, "newsgroup descriptions",
     "Descriptions in form \"group description\""
 };
 static LISTINFO		INFOmoderators = {
-    "moderators", _PATH_MODERATORS, NULL, false, "moderator patterns",
+    "MODERATORS", INN_PATH_MODERATORS, NULL, false, "moderator patterns",
     "Newsgroup moderators in form \"group-pattern:mail-address-pattern\""
 };
 static LISTINFO		INFOschema = {
-    "overview.fmt", NULL, cmd_list_schema, true, "overview format",
+    "OVERVIEW.FMT", NULL, cmd_list_schema, true, "overview format",
     "Order of fields in overview database"
 };
 static LISTINFO		INFOmotd = {
-    "motd", _PATH_MOTD, NULL, false, "motd",
+    "MOTD", INN_PATH_MOTD, NULL, false, "motd",
     "Message of the day text"
 };
 
@@ -68,9 +68,9 @@ static LISTINFO *info[] = {
     &INFOactive,
     &INFOactivetimes,
     &INFOdistribs,
+    &INFOheaders,
     &INFOsubs,
     &INFOdistribpats,
-    &INFOextensions,
     &INFOgroups,
     &INFOmoderators,
     &INFOschema,
@@ -79,15 +79,15 @@ static LISTINFO *info[] = {
 
 
 /*
-**  List the overview schema
+**  List the overview schema (standard and extra fields).
 */
 static void
-cmd_list_schema(LISTINFO *lp)
+cmd_list_schema(LISTINFO *lp, int ac UNUSED, char *av[] UNUSED)
 {
     const struct cvector *standard;
     unsigned int i;
 
-    Reply("%d %s.\r\n", NNTP_LIST_FOLLOWS_VAL, lp->Format);
+    Reply("%d %s\r\n", NNTP_OK_LIST, lp->Format);
     standard = overview_fields();
     for (i = 0; i < standard->count; ++i) {
 	Printf("%s:\r\n", standard->strings[i]);
@@ -100,15 +100,28 @@ cmd_list_schema(LISTINFO *lp)
 
 
 /*
-**  List supported extensions
+**  List supported headers and metadata information.
 */
 static void
-cmd_list_extensions(LISTINFO *lp)
+cmd_list_headers(LISTINFO *lp, int ac, char *av[])
 {
-    Reply("%d %s.\r\n", NNTP_LIST_FOLLOWS_VAL, lp->Format);
-    if (PERMauthorized != true)
-        Printf("AUTHINFO USER\r\n");
-    Printf("LISTGROUP\r\n");
+    bool range;
+
+    range = (ac > 2 && strcasecmp(av[2], "RANGE") == 0);
+    
+    if (ac > 2 && (strcasecmp(av[2], "MSGID") != 0)
+        && !range) {
+        Reply("%d Syntax error in arguments\r\n", NNTP_ERR_SYNTAX);
+        return;
+    }
+    Reply("%d %s\r\n", NNTP_OK_LIST, lp->Format);
+    Printf(":\r\n");
+    if (range) {
+        /* These information are only known by the overview system,
+         * and are only accessible with a range. */
+        Printf(":bytes\r\n");
+        Printf(":lines\r\n");
+    }
     Printf(".\r\n");
 }
 
@@ -124,16 +137,24 @@ static bool
 CMD_list_single(char *group)
 {
     char *grplist[2] = { NULL, NULL };
-    int lo, hi, flag;
+    int lo, hi, count, flag;
 
     if (PERMspecified) {
         grplist[0] = group;
         if (!PERMmatch(PERMreadlist, grplist))
             return false;
     }
-    if (OVgroupstats(group, &lo, &hi, NULL, &flag) && flag != '=') {
-        Reply("%d %s.\r\n", NNTP_LIST_FOLLOWS_VAL, INFOactive.Format);
-        Printf("%s %010u %010u %c\r\n.\r\n", group, hi, lo, flag);
+    if (OVgroupstats(group, &lo, &hi, &count, &flag) && flag != '=') {
+        /* Convert flags to standardized ones, if possible. */
+        if (flag == 'j' || flag == 'x')
+            flag = 'n';
+        /* When a newsgroup is empty, the high water mark should be one less
+         * than the low water mark according to RFC 3977. */
+        if (count == 0)
+            lo = hi + 1;
+        Reply("%d %s\r\n", NNTP_OK_LIST, INFOactive.Format);
+        Printf("%s %010u %010u %c\r\n", group, hi, lo, flag);
+        Printf(".\r\n");
         return true;
     }
     return false;
@@ -141,7 +162,7 @@ CMD_list_single(char *group)
 
 
 /*
-**  List active newsgroups, newsgroup descriptions, and distributions.
+**  Main LIST function.
 */
 void
 CMDlist(int ac, char *av[])
@@ -158,6 +179,8 @@ CMDlist(int ac, char *av[])
     unsigned int i;
 
     p = av[1];
+    /* LIST ACTIVE is the default LIST command.  If a keyword is provided,
+     * we check whether it is defined. */
     if (p == NULL) {
 	lp = &INFOactive;
     } else {
@@ -169,52 +192,59 @@ CMDlist(int ac, char *av[])
 	    }
 	}
     }
+    /* If no defined LIST keyword is found, we return. */
     if (lp == NULL) {
-	Reply("%s\r\n", NNTP_SYNTAX_USE);
-	return;
+        Reply("%d Unknown LIST keyword\r\n", NNTP_ERR_SYNTAX);
+        return;
     }
+
     if (lp == &INFOactive) {
 	if (ac == 3) {
 	    wildarg = av[2];
+            /* No need to parse the active file for a single group. */
             if (CMD_list_single(wildarg))
 		return;
 	}
-    } else if (lp == &INFOgroups || lp == &INFOactivetimes) {
+    } else if (lp == &INFOgroups || lp == &INFOactivetimes
+               || lp == &INFOheaders) {
 	if (ac == 3)
 	    wildarg = av[2];
     }
-
+    /* Three arguments can be passed only when ACTIVE, ACTIVE.TIMES,
+     * HEADERS or NEWSGROUPS keywords are used. */
     if (ac > 2 && !wildarg) {
-	Reply("%s\r\n", NNTP_SYNTAX_USE);
-	return;
+        Reply("%d Unexpected wildmat\r\n", NNTP_ERR_SYNTAX);
+        return;
     }
 
+    /* If a function is provided for the given keyword, we call it. */
     if (lp->impl != NULL) {
-	lp->impl(lp);
+	lp->impl(lp, ac, av);
 	return;
     }
 
     path = innconf->pathetc;
+    /* The active, active.times and newsgroups files are in pathdb. */
     if ((strstr(lp->File, "active") != NULL) ||
 	(strstr(lp->File, "newsgroups") != NULL))
 	path = innconf->pathdb;
-    if (strchr(lp->File, '/') != NULL)
-	path = "";
     path = concatpath(path, lp->File);
     qp = QIOopen(path);
     free(path);
     if (qp == NULL) {
-        Reply("%d No list of %s available.\r\n",
-              NNTP_TEMPERR_VAL, lp->Items);
-	    if (lp->Required || errno != ENOENT) {
-            syslog(L_ERROR, "%s cant fopen %s %m", ClientHost, lp->File);
+        Reply("%d No list of %s available\r\n",
+              NNTP_ERR_UNAVAILABLE, lp->Items);
+        /* Only the active and newsgroups files are required. */
+        if (lp->Required || errno != ENOENT) {
+            /* %m outputs strerror(errno). */
+            syslog(L_ERROR, "%s can't fopen %s %m", Client.host, lp->File);
         }
         return;
     }
 
-    Reply("%d %s.\r\n", NNTP_LIST_FOLLOWS_VAL, lp->Format);
+    Reply("%d %s\r\n", NNTP_OK_LIST, lp->Format);
     if (!PERMspecified) {
-	/* Optmize for unlikely case of no permissions and false default. */
+	/* Optimize for unlikely case of no permissions and false default. */
 	QIOclose(qp);
 	Printf(".\r\n");
 	return;
@@ -230,11 +260,11 @@ CMDlist(int ac, char *av[])
 	    continue;
 	}
 	if (p[0] == '.' && p[1] == '\0') {
-	    syslog(L_ERROR, "%s single dot in %s", ClientHost, lp->File);
+	    syslog(L_ERROR, "%s single dot in %s", Client.host, lp->File);
 	    continue;
 	}
-	/* matching patterns against patterns is not that
-	   good but it's better than nothing ... */
+	/* Matching patterns against patterns is not that
+	 * good but it is better than nothing... */
 	if (lp == &INFOdistribpats) {
 	    if (*p == '\0' || *p == '#' || *p == ';' || *p == ' ')
 		continue;
@@ -266,16 +296,29 @@ CMDlist(int ac, char *av[])
 		break;
 	    }
 	}
-	      
+
+        /* Check whether the reader has access to the newsgroup. */
 	if (PERMspecified) {
 	    grplist[0] = p;
 	    if (!PERMmatch(PERMreadlist, grplist))
 		continue;
 	}
+        /* Check whether the newsgroup matches the wildmat pattern,
+         * if given. */
 	if (wildarg && !uwildmat(p, wildarg))
 	    continue;
+
 	if (savec != '\0')
 	    *save = savec;
+
+        if (lp == &INFOactive) {
+            /* Convert flags to standardized ones, if possible. */
+            if ((q = strrchr(p, ' ')) != NULL) {
+                q++;
+                if (*q == 'j' || *q == 'x')
+                    *q = 'n';
+            }
+        }
 	Printf("%s\r\n", p);
     }
     QIOclose(qp);
