@@ -1,6 +1,6 @@
-/*  $Id: nnrpd.h,v 1.76.2.1 2004/01/12 05:18:44 rra Exp $
+/*  $Id: nnrpd.h 8266 2008-12-27 13:39:23Z iulius $
 **
-**  Net News Reading Protocol server.
+**  NetNews Reading Protocol server.
 */
 
 #include "config.h"
@@ -14,38 +14,29 @@
 #include <sys/stat.h>
 
 #include "inn/qio.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 #include "nntp.h"
-#include "paths.h"
-#include "storage.h"
+#include "inn/paths.h"
+#include "inn/storage.h"
 #include "inn/vector.h"
 #include "inn/timer.h"
 
-/*
-**  Maximum input line length, sigh.
-*/
-#define ART_LINE_LENGTH		1000
-#define ART_LINE_MALLOC		1024
-#define ART_MAX			1024
-
-
-/*
-**  Some convenient shorthands.
-*/
-typedef struct in_addr	INADDR;
-
+#ifdef HAVE_SASL
+#include <sasl/sasl.h>
+#include <sasl/saslutil.h>
+#endif
 
 /*
 **  A range of article numbers.
 */
 typedef struct _ARTRANGE {
-    int		Low;
-    int		High;
+    ARTNUM	Low;
+    ARTNUM	High;
 } ARTRANGE;
 
 /*
-** access configuration for each readers
- */
+**  Access configuration for each reader.
+*/
 typedef struct _ACCESSGROUP {
     char *name;
     char *key;
@@ -104,7 +95,7 @@ typedef enum _READTYPE {
 
 
 /*
-** Structure used by line_read to keep track of what's been read
+**  Structure used by line_read to keep track of what has been read.
 */
 struct line {
     char *start;
@@ -113,6 +104,22 @@ struct line {
     size_t allocated;
 };
 
+
+/*
+**  Information about the currently connected client.  Eventually, this struct
+**  may become the repository for all nnrpd global state, or at least the
+**  state that relates to a particular client connection.
+*/
+struct client {
+    char host[SMBUF];
+    char ip[INET6_ADDRSTRLEN];
+    unsigned short port;
+    char serverhost[SMBUF];
+    char serverip[INET6_ADDRSTRLEN];
+    unsigned short serverport;
+};
+
+
 /*
 **  Information about the schema of the news overview files.
 */
@@ -128,11 +135,11 @@ typedef struct _ARTOVERFIELD {
 */
 enum timer {
     TMR_IDLE = TMR_APPLICATION, /* Server is completely idle. */
-    TMR_NEWNEWS,                /* Executing NEWNEWS command */
-    TMR_READART,                /* Reading an article (SMretrieve) */
-    TMR_CHECKART,               /* Checking an article (ARTinstorebytoken) */
-    TMR_NNTPREAD,               /* Reading from the peer */
-    TMR_NNTPWRITE,              /* Writing to the peer */
+    TMR_NEWNEWS,                /* Executing NEWNEWS command. */
+    TMR_READART,                /* Reading an article (SMretrieve). */
+    TMR_CHECKART,               /* Checking an article (ARTinstorebytoken). */
+    TMR_NNTPREAD,               /* Reading from the peer. */
+    TMR_NNTPWRITE,              /* Writing to the peer. */
     TMR_MAX
 };
 
@@ -143,80 +150,67 @@ enum timer {
 #endif	/* defined(MAINLINE) */
 
 EXTERN bool	PERMauthorized;
+EXTERN bool     PERMcanauthenticate;
+#ifdef HAVE_SSL
+EXTERN bool     PERMcanauthenticatewithoutSSL;
+#endif
 EXTERN bool	PERMcanpost;
+EXTERN bool     PERMcanpostgreeting;
 EXTERN bool	PERMcanread;
 EXTERN bool	PERMneedauth;
 EXTERN bool	PERMspecified;
+EXTERN bool     PERMgroupmadeinvalid;
 EXTERN ACCESSGROUP	*PERMaccessconf;
 EXTERN bool	Tracing;
 EXTERN bool 	Offlinepost;
 EXTERN bool 	initialSSL;
 EXTERN char	**PERMreadlist;
 EXTERN char	**PERMpostlist;
-EXTERN char	ClientHost[SMBUF];
-EXTERN char     ServerHost[SMBUF];
+EXTERN struct client Client;
 EXTERN char	Username[SMBUF];
-#ifdef HAVE_INET6
-EXTERN char     ClientIpString[INET6_ADDRSTRLEN];
-EXTERN char     ServerIpString[INET6_ADDRSTRLEN];
-#else
-EXTERN char     ClientIpString[20];
-EXTERN char     ServerIpString[20];
-#endif
-EXTERN int	ClientPort;
-EXTERN int	ServerPort;
-EXTERN char	LogName[256] ;
-#ifdef HAVE_SSL
-EXTERN bool	ClientSSL;
-#endif
 extern char	*ACTIVETIMES;
 extern char	*HISTORY;
 extern char	*ACTIVE;
 extern char	*NEWSGROUPS;
 extern char	*NNRPACCESS;
-extern char	NOACCESS[];
-EXTERN int	SPOOLlen;
-EXTERN char	PERMpass[SMBUF];
 EXTERN char	PERMuser[SMBUF];
 EXTERN FILE	*locallog;
-EXTERN int	ARTnumber;	/* Current article number */
-EXTERN int	ARThigh;	/* Current high number for group */
-EXTERN int	ARTlow;		/* Current low number for group */
-EXTERN long	ARTcount;	/* Current number of articles in group */
-EXTERN long	MaxBytesPerSecond; /* maximum bytes per sec a client can use, defaults to 0 */
+EXTERN ARTNUM	ARTnumber;	/* Current article number. */
+EXTERN ARTNUM	ARThigh;	/* Current high number for group. */
+EXTERN ARTNUM	ARTlow;		/* Current low number for group. */
+EXTERN unsigned long	ARTcount;	/* Number of articles in group. */
+EXTERN long	MaxBytesPerSecond; /* Maximum bytes per sec a client can use, defaults to 0. */
 EXTERN long	ARTget;
 EXTERN long	ARTgettime;
 EXTERN long	ARTgetsize;
-EXTERN long	OVERcount;	/* number of XOVER commands */
-EXTERN long	OVERhit;	/* number of XOVER records found in .overview */
-EXTERN long	OVERmiss;	/* number of XOVER records found in articles */
-EXTERN long	OVERtime;	/* number of ms spent sending XOVER data */
-EXTERN long	OVERsize;	/* number of bytes of XOVER data sent	*/
-EXTERN long	OVERdbz;	/* number of ms spent reading dbz data	*/
-EXTERN long	OVERseek;	/* number of ms spent seeking history	*/
-EXTERN long	OVERget;	/* number of ms spent reading history	*/
-EXTERN long	OVERartcheck;	/* number of ms spent article check	*/
+EXTERN long	OVERcount;	/* Number of (X)OVER commands. */
+EXTERN long	OVERhit;	/* Number of (X)OVER records found in overview. */
+EXTERN long	OVERmiss;	/* Number of (X)OVER records found in articles. */
+EXTERN long	OVERtime;	/* Number of ms spent sending (X)OVER data. */
+EXTERN long	OVERsize;	/* Number of bytes of (X)OVER data sent. */
+EXTERN long	OVERdbz;	/* Number of ms spent reading dbz data. */
+EXTERN long	OVERseek;	/* Number of ms spent seeking history. */
+EXTERN long	OVERget;	/* Number of ms spent reading history. */
+EXTERN long	OVERartcheck;	/* Number of ms spent article check. */
 EXTERN double	IDLEtime;
-EXTERN long	GRParticles;
+EXTERN unsigned long	GRParticles;
 EXTERN long	GRPcount;
 EXTERN char	*GRPcur;
 EXTERN long	POSTreceived;
 EXTERN long	POSTrejected;
 
 EXTERN bool     BACKOFFenabled;
-EXTERN long     ClientIpAddr;
 EXTERN char	*VirtualPath;
 EXTERN int	VirtualPathlen;
 EXTERN struct history *History;
 EXTERN struct line NNTPline;
 EXTERN struct vector *OVextra;
 EXTERN int	overhdr_xref;
-EXTERN bool	LLOGenable;
+EXTERN bool     LLOGenable;
 
 extern const char	*ARTpost(char *article, char *idbuff, bool ihave,
 				 bool *permanent);
 extern void		ARTclose(void);
-extern bool		ARTreadschema(void);
 extern int		TrimSpaces(char *line);
 extern char		*Glom(char **av);
 extern int		Argify(char *line, char ***argvp);
@@ -233,56 +227,78 @@ extern bool		NGgetlist(char ***argvp, char *list);
 extern bool		PERMartok(void);
 extern void		PERMgetaccess(char *nnrpaccess);
 extern void		PERMgetpermissions(void);
-extern void		PERMlogin(char *uname, char *pass, char *errorstr);
+extern void		PERMlogin(char *uname, char *pass, int* code, char *errorstr);
 extern bool		PERMmatch(char **Pats, char **list);
 extern bool		ParseDistlist(char ***argvp, char *list);
 extern void 		SetDefaultAccess(ACCESSGROUP*);
 extern void		Reply(const char *fmt, ...);
 extern void             Printf(const char *fmt, ...);
 
-extern void		CMDauthinfo  (int ac, char** av);
-extern void		CMDdate      (int ac, char** av);
-extern void		CMDfetch     (int ac, char** av);
-extern void		CMDgroup     (int ac, char** av);
-extern void		CMDhelp      (int ac, char** av);
-extern void		CMDlist      (int ac, char** av);
-extern void		CMDmode      (int ac, char** av);
-extern void		CMDnewgroups (int ac, char** av);
-extern void		CMDnewnews   (int ac, char** av);
-extern void		CMDnextlast  (int ac, char** av);
-extern void		CMDpost      (int ac, char** av);
-extern void		CMDxgtitle   (int ac, char** av);
-extern void		CMDxover     (int ac, char** av);
-extern void		CMDpat       (int ac, char** av);
-extern void		CMDxpath     (int ac, char** av);
-extern void		CMD_unimp    (int ac, char** av);
+extern void             CMDauthinfo     (int ac, char** av);
+extern void             CMDcapabilities (int ac, char** av);
+extern void             CMDdate         (int ac, char** av);
+extern void             CMDfetch        (int ac, char** av);
+extern void             CMDgroup        (int ac, char** av);
+extern void             CMDhelp         (int ac, char** av);
+extern void             CMDlist         (int ac, char** av);
+extern void             CMDmode         (int ac, char** av);
+extern void             CMDnewgroups    (int ac, char** av);
+extern void             CMDnewnews      (int ac, char** av);
+extern void             CMDnextlast     (int ac, char** av);
+extern void             CMDover         (int ac, char** av);
+extern void             CMDpost         (int ac, char** av);
+extern void             CMDquit         (int ac, char** av);
+extern void             CMDxgtitle      (int ac, char** av);
+extern void             CMDpat          (int ac, char** av);
+extern void             CMD_unimp       (int ac, char** av);
 #ifdef HAVE_SSL
-extern void		CMDstarttls  (int ac, char** av);
+extern void             CMDstarttls     (int ac, char** av);
 #endif
 
+extern bool CMDgetrange(int ac, char *av[], ARTRANGE *rp, bool *DidReply);
 
+/*
+**  Run a resolver or authenticator.  The directory is where to look for the
+**  command if not fully qualified.  username and password may be NULL to run
+**  resolvers.
+*/
+char *auth_external(struct client *, const char *command,
+                    const char *directory, const char *username,
+                    const char *password);
+
+void write_buffer(const char *buff, ssize_t len);
 
 extern char *HandleHeaders(char *article);
 extern bool ARTinstorebytoken(TOKEN token);
 
-extern int TrackClient(char *client, char* user);
+extern int TrackClient(char *client, char* user, size_t len);
 
 #ifdef  DO_PERL
 extern void loadPerl(void);
 extern void perlAccess(char *user, struct vector *access_vec);
-extern int perlAuthenticate(char *user, char *passwd, char *errorstring, char*newUser);
+extern void perlAuthenticate(char *user, char *passwd, int *code, char *errorstring, char*newUser);
 extern void perlAuthInit(void);
 #endif /* DO_PERL */
 
 #ifdef	DO_PYTHON
 extern bool PY_use_dynamic;
 
-int PY_authenticate(char *path, char *Username, char *Password, char *errorstring, char *newUser);
+void PY_authenticate(char *path, char *Username, char *Password, int *code, char *errorstring, char *newUser);
 void PY_access(char* path, struct vector *access_vec, char *Username);
+void PY_close_python(void);
 int PY_dynamic(char *Username, char *NewsGroup, int PostFlag, char **reply_message);
 void PY_dynamic_init (char* file);
 #endif	/* DO_PYTHON */
 
 void line_free(struct line *);
 void line_init(struct line *);
-READTYPE line_read(struct line *, int, const char **, size_t *);
+READTYPE line_read(struct line *, int, const char **, size_t *, size_t *);
+
+#ifdef HAVE_SASL
+extern sasl_conn_t *sasl_conn;
+extern int sasl_ssf, sasl_maxout;
+extern sasl_callback_t sasl_callbacks[];
+
+void SASLauth(int ac, char *av[]);
+void SASLnewserver(void);
+#endif /* HAVE_SASL */
