@@ -1,4 +1,4 @@
-/*  $Revision: 7450 $
+/*  $Id: post.c 8444 2009-05-04 19:44:55Z iulius $
 **
 **  Check article, send it to the local server.
 */
@@ -7,7 +7,7 @@
 
 #include "inn/innconf.h"
 #include "nnrpd.h"
-#include "ov.h"
+#include "inn/ov.h"
 #include "post.h"
 
 #define FLUSH_ERROR(F)		(fflush((F)) == EOF || ferror((F)))
@@ -25,7 +25,7 @@ static const char * const BadDistribs[] = {
 };
 
 HEADER Table[] = {
-    /*  Name                    Canset  Type    Size  Value */
+    /*  Name                    CanSet  Type    Size  Value */
     {   "Path",                 true,   HTstd,  0,    NULL,    NULL, 0 },
     {   "From",                 true,   HTreq,  0,    NULL,    NULL, 0 },
     {   "Newsgroups",           true,   HTreq,  0,    NULL,    NULL, 0 },
@@ -59,22 +59,21 @@ HEADER Table[] = {
     {   "Posted",               false,  HTobs,  0,    NULL,    NULL, 0 },
     {   "Posting-Version",      false,  HTobs,  0,    NULL,    NULL, 0 },
     {   "Relay-Version",        false,  HTobs,  0,    NULL,    NULL, 0 },
-    {   "Cc",                   true, HTstd,  0,    NULL,    NULL, 0 },
-    {   "Bcc",                  true, HTstd,  0,    NULL,    NULL, 0 },
-    {   "To",                   true, HTstd,  0,    NULL,    NULL, 0 },
+    {   "Cc",                   true,   HTstd,  0,    NULL,    NULL, 0 },
+    {   "Bcc",                  true,   HTstd,  0,    NULL,    NULL, 0 },
+    {   "To",                   true,   HTstd,  0,    NULL,    NULL, 0 },
 };
 
 HEADER *EndOfTable = ARRAY_END(Table);
 
-
 
-/* Join() and MaxLength() are taken from innd.c */
 /*
 **  Turn any \r or \n in text into spaces.  Used to splice back multi-line
 **  headers into a single line.
+**  Taken from innd.c.
 */
 static char *
-Join(char	*text)
+Join(char *text)
 {
     char	*p;
 
@@ -84,9 +83,11 @@ Join(char	*text)
     return text;
 }
 
+
 /*
-**  Return a short name that won't overrun our bufer or syslog's buffer.
+**  Return a short name that won't overrun our buffer or syslog's buffer.
 **  q should either be p, or point into p where the "interesting" part is.
+**  Taken from innd.c.
 */
 static char *
 MaxLength(char *p, char *q)
@@ -94,6 +95,13 @@ MaxLength(char *p, char *q)
     static char		buff[80];
     unsigned int	i;
 
+
+    /* Return an empty string when p is NULL. */
+    if (p == NULL) {
+        *buff = '\0';
+        return buff;
+    }
+
     /* Already short enough? */
     i = strlen(p);
     if (i < sizeof buff - 1)
@@ -104,7 +112,7 @@ MaxLength(char *p, char *q)
 	q = p;
 
     /* Simple case of just want the begining? */
-    if ((size_t)(q - p) < sizeof(buff) - 4) {
+    if (q == NULL || (size_t)(q - p) < sizeof(buff) - 4) {
 	strlcpy(buff, p, sizeof(buff) - 3);
         strlcat(buff, "...", sizeof(buff));
     } else if ((p + i) - q < 10) {
@@ -113,7 +121,7 @@ MaxLength(char *p, char *q)
         strlcat(buff, "...", sizeof(buff) - 10);
         strlcat(buff, &p[i - 10], sizeof(buff));
     } else {
-	/* Not in last 10 bytes, so use double elipses. */
+	/* Not in last 10 bytes, so use double ellipses. */
         strlcpy(buff, p, sizeof(buff) - 16);
         strlcat(buff, "...", sizeof(buff) - 13);
         strlcat(buff, &q[-5], sizeof(buff) - 3);
@@ -121,6 +129,8 @@ MaxLength(char *p, char *q)
     }
     return Join(buff);
 }
+
+
 /*
 **  Trim trailing spaces, return pointer to first non-space char.
 */
@@ -176,18 +186,18 @@ StripOffHeaders(char *article)
 	     && p[hp->Size] == ':'
 	     && strncasecmp(p, hp->Name, hp->Size) == 0) {
 		if (hp->Type == HTobs) {
-		    snprintf(Error, sizeof(Error), "Obsolete \"%s\" header",
+		    snprintf(Error, sizeof(Error), "Obsolete %s: header",
                              hp->Name);
 		    return NULL;
 		}
 		if (hp->Value) {
-		    snprintf(Error, sizeof(Error), "Duplicate \"%s\" header",
+		    snprintf(Error, sizeof(Error), "Duplicate %s: header",
                              hp->Name);
 		    return NULL;
 		}
 		hp->Value = &p[hp->Size + 1];
 		/* '\r\n' is replaced with '\n', and unnecessary to consider
-		   '\r' */
+		 *  '\r'. */
 		for (q = &p[hp->Size + 1]; ISWHITE(*q) || *q == '\n'; q++)
 		    continue;
 		hp->Body = q;
@@ -217,7 +227,7 @@ StripOffHeaders(char *article)
     return p + 1;
 }
 
-
+
 
 /*
 **  Check the control message, and see if it's legit.  Return pointer to
@@ -267,7 +277,7 @@ CheckControl(char *ctrl)
 
 
 /*
-**  Check the Distribution header, and exit on error.
+**  Check the Distribution: header, and exit on error.
 */
 static const char *
 CheckDistribution(char *p)
@@ -276,7 +286,7 @@ CheckDistribution(char *p)
     const char * const *dp;
 
     if ((p = strtok(p, SEPS)) == NULL)
-	return "Can't parse Distribution line.";
+	return "Can't parse Distribution: header";
     do {
 	for (dp = BadDistribs; *dp; dp++)
 	    if (uwildmat(p, *dp)) {
@@ -307,25 +317,20 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
     static char		*newpath = NULL;
     HEADER		*hp;
     char		*p;
-    time_t		t;
+    time_t		t, now;
     struct tm		*gmt;
-    TIMEINFO		Now;
     const char          *error;
     pid_t               pid;
     bool		addvirtual = false;
 
-    /* Various things need Now to be set. */
-    if (GetTimeInfo(&Now) < 0) {
-        snprintf(Error, sizeof(Error), "Can't get the time, %s",
-                 strerror(errno));
-        return Error;
-    }
+    /* Get the current time, used for creating and checking dates. */
+    now = time(NULL);
 
     /* Do some preliminary fix-ups. */
     for (hp = Table; hp < ARRAY_END(Table); hp++) {
 	if (!ihave && !hp->CanSet && hp->Value) {
 	    snprintf(Error, sizeof(Error),
-		     "Can't set system \"%s\" header", hp->Name);
+		     "Can't set system %s: header", hp->Name);
 	    return Error;
 	}
 	if (hp->Value) {
@@ -336,29 +341,29 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
     }
 
     /* If authorized, add the header based on our info.  If not authorized,
-       zap the Sender so we don't put out unauthenticated data. */
+     * zap the Sender: header so we don't put out unauthenticated data. */
     if (PERMaccessconf->nnrpdauthsender) {
 	if (PERMauthorized && PERMuser[0] != '\0') {
             p = strchr(PERMuser, '@');
             if (p == NULL) {
                 snprintf(sendbuff, sizeof(sendbuff), "%s@%s", PERMuser,
-                         ClientHost);
+                         Client.host);
             } else {
                 snprintf(sendbuff, sizeof(sendbuff), "%s", PERMuser);
             }
 	    HDR_SET(HDR__SENDER, sendbuff);
 	} else {
-	    HDR_SET(HDR__SENDER, NULL);
+	    HDR_CLEAR(HDR__SENDER);
 	}
     }
 
-    /* Set Date.  datebuff is used later for NNTP-Posting-Date, so we have
-       to set it and it has to be the UTC date. */
+    /* Set the Date: header.  datebuff is used later for NNTP-Posting-Date:,
+     *  so we have to set it and it has to be the UTC date. */
     if (!makedate(-1, false, datebuff, sizeof(datebuff)))
-        return "Can't generate date header";
+        return "Can't generate Date: header";
     if (HDR(HDR__DATE) == NULL) {
 	if (ihave)
-	    return "Missing \"Date\" header";
+	    return "Missing Date: header";
         if (PERMaccessconf->localtime) {
             if (!makedate(-1, true, localdatebuff, sizeof(localdatebuff)))
                 return "Can't generate local date header";
@@ -367,13 +372,14 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
 	    HDR_SET(HDR__DATE, datebuff);
 	}
     } else {
-	if ((t = parsedate(HDR(HDR__DATE), &Now)) == -1)
-	    return "Can't parse \"Date\" header";
-	if (t > Now.time + DATE_FUZZ)
+        t = parsedate_rfc2822_lax(HDR(HDR__DATE));
+	if (t == (time_t) -1)
+	    return "Can't parse Date: header";
+	if (t > now + DATE_FUZZ)
 	    return "Article posted in the future";
     }
 
-    /* Newsgroups are checked later. */
+    /* Newsgroups: is checked later. */
 
     if (HDR(HDR__CONTROL)) {
 	if ((error = CheckControl(HDR(HDR__CONTROL))) != NULL)
@@ -381,7 +387,7 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
     } else {
 	p = HDR(HDR__SUBJECT);
 	if (p == NULL)
-	    return "Required \"Subject\" header is missing";
+	    return "Required Subject: header is missing";
         if (strncmp(p, "cmsg ", 5) == 0) {
             HDR_SET(HDR__CONTROL, p + 5);
             if ((error = CheckControl(HDR(HDR__CONTROL))) != NULL)
@@ -389,37 +395,21 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
         }
     }
 
-    /* Set Message-ID */
+    /* Set the Message-ID: header. */
     if (HDR(HDR__MESSAGEID) == NULL) {
 	if (ihave)
-	    return "Missing \"Message-ID\" header";
+	    return "Missing Message-ID: header";
 	HDR_SET(HDR__MESSAGEID, idbuff);
     }
 
-    /* Set Path */
-    if (HDR(HDR__PATH) == NULL) {
- 	if (ihave)
-	    return "Missing \"Path\" header";
+    /* Set the Path: header. */
+    if (HDR(HDR__PATH) == NULL && ihave)
+        return "Missing Path: header";
+    if (HDR(HDR__PATH) == NULL || PERMaccessconf->strippath) {
 	/* Note that innd will put host name here for us. */
-	HDR_SET(HDR__PATH, PATHMASTER);
+	HDR_SET(HDR__PATH, (char *) PATHMASTER);
 	if (VirtualPathlen > 0)
 	    addvirtual = true;
-    } else if (PERMaccessconf->strippath) {
-	/* Here's where to do Path changes for new Posts. */
-	if ((p = strrchr(HDR(HDR__PATH), '!')) != NULL) {
-	    p++;
-	    if (*p == '\0') {
-		HDR_SET(HDR__PATH, PATHMASTER);
-		if (VirtualPathlen > 0)
-		    addvirtual = true;
-	    } else {
-		HDR_SET(HDR__PATH, p);
-		if ((VirtualPathlen > 0) &&
-		    strcmp(p, PERMaccessconf->pathhost) != 0)
-		    addvirtual = true;
-	    }
-	} else if (VirtualPathlen > 0)
-	    addvirtual = true;
     } else {
 	if ((VirtualPathlen > 0) &&
 	    (p = strchr(HDR(HDR__PATH), '!')) != NULL) {
@@ -436,19 +426,18 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
         newpath = concat(VirtualPath, HDR(HDR__PATH), (char *) 0);
 	HDR_SET(HDR__PATH, newpath);
     }
-    
 
-    /* Reply-To; left alone. */
-    /* Sender; set above. */
+    /* Reply-To: is left alone. */
+    /* Sender: is set above. */
 
-    /* Check Expires. */
-    if (HDR(HDR__EXPIRES) && parsedate(HDR(HDR__EXPIRES), &Now) == -1)
-	return "Can't parse \"Expires\" header";
+    /* Check the Expires: header. */
+    if (HDR(HDR__EXPIRES) && parsedate_rfc2822_lax(HDR(HDR__EXPIRES)) == -1)
+	return "Can't parse Expires: header";
 
-    /* References; left alone. */
-    /* Control; checked above. */
+    /* References: is left alone. */
+    /* Control! is checked above. */
 
-    /* Distribution. */
+    /* Check the Distribution: header. */
     if ((p = HDR(HDR__DISTRIBUTION)) != NULL) {
 	p = xstrdup(p);
 	error = CheckDistribution(p);
@@ -457,50 +446,50 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
 	    return error;
     }
 
-    /* Set Organization */
+    /* Set the Organization: header. */
     if (!ihave && HDR(HDR__ORGANIZATION) == NULL
      && (p = PERMaccessconf->organization) != NULL) {
 	strlcpy(orgbuff, p, sizeof(orgbuff));
 	HDR_SET(HDR__ORGANIZATION, orgbuff);
     }
 
-    /* Keywords; left alone. */
-    /* Summary; left alone. */
-    /* Approved; left alone. */
+    /* Keywords: is left alone. */
+    /* Summary: is left alone. */
+    /* Approved: is left alone. */
 
-    /* Set Lines */
+    /* Set the Lines: header. */
     if (!ihave) {
 	snprintf(linebuff, sizeof(linebuff), "%d", linecount);
 	HDR_SET(HDR__LINES, linebuff);
     }
 
-    /* Supersedes; left alone. */
+    /* Supersedes: is left alone. */
 
-    /* NNTP-Posting host; set. */
+    /* Set the NNTP-Posting-Host: header. */
     if (!ihave && PERMaccessconf->addnntppostinghost) 
-    HDR_SET(HDR__NNTPPOSTINGHOST, ClientHost);
-    /* NNTP-Posting-Date - not in RFC (yet) */
+    HDR_SET(HDR__NNTPPOSTINGHOST, Client.host);
+    
+    /* Set the NNTP-Posting-Date: header. */
     if (!ihave && PERMaccessconf->addnntppostingdate)
     HDR_SET(HDR__NNTPPOSTINGDATE, datebuff);
 
-    /* X-Trace; set */
-    t = time((time_t *)NULL) ;
+    /* Set the X-Trace: header. */
     pid = (long) getpid() ;
-    if ((gmt = gmtime(&Now.time)) == NULL)
+    if ((gmt = gmtime(&now)) == NULL)
 	return "Can't get the time";
     if (VirtualPathlen > 0)
 	p = PERMaccessconf->domain;
     else
 	if ((p = GetFQDN(PERMaccessconf->domain)) == NULL)
-	    p = "unknown";
+	    p = (char *) "unknown";
     snprintf(tracebuff, sizeof(tracebuff),
              "%s %ld %ld %s (%d %3.3s %d %02d:%02d:%02d GMT)",
-             p, (long) t, (long) pid, ClientIpString,
+             p, (long) now, (long) pid, Client.ip,
              gmt->tm_mday, &MONTHS[3 * gmt->tm_mon], 1900 + gmt->tm_year,
              gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
     HDR_SET(HDR__XTRACE, tracebuff);
 
-    /* X-Complaints-To; set */
+    /* Set the X-Complaints-To: header. */
     if ((p = PERMaccessconf->complaints) != NULL)
 	snprintf (complaintsbuff, sizeof(complaintsbuff), "%s", p);
     else {
@@ -515,17 +504,18 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
     }
     HDR_SET(HDR__XCOMPLAINTSTO, complaintsbuff);
 
-    /* Clear out some headers that should not be here */
+    /* Clear out some headers that should not be here. */
     if (!ihave && PERMaccessconf->strippostcc) {
-	HDR_SET(HDR__CC, NULL);
-	HDR_SET(HDR__BCC, NULL);
-	HDR_SET(HDR__TO, NULL);
+	HDR_CLEAR(HDR__CC);
+	HDR_CLEAR(HDR__BCC);
+	HDR_CLEAR(HDR__TO);
     }
+
     /* Now make sure everything is there. */
     for (hp = Table; hp < ARRAY_END(Table); hp++)
 	if (hp->Type == HTreq && hp->Value == NULL) {
 	    snprintf(Error, sizeof(Error),
-                     "Required \"%s\" header is missing", hp->Name);
+                     "Missing required %s: header", hp->Name);
 	    return Error;
 	}
 
@@ -538,7 +528,7 @@ ProcessHeaders(int linecount, char *idbuff, bool ihave)
 **  but reasonably effective for catching neophyte's mistakes.  Son-of-1036
 **  says:
 **
-**      NOTE: While encouraging trimming is desirable, the 50% rule imposed
+**      NOTE:  While encouraging trimming is desirable, the 50% rule imposed
 **      by some old posting agents is both inadequate and counterproductive.
 **      Posters do not respond to it by being more selective about quoting;
 **      they respond by padding short responses, or by using different
@@ -581,7 +571,7 @@ CheckIncludedText(const char *p, int lines)
     return NULL;
 }
 
-
+
 
 /*
 **  Try to mail an article to the moderator of the group.
@@ -609,7 +599,7 @@ MailArticle(char *group, char *article)
     /* Now build up the command (ignore format/argument mismatch errors,
      * in case %s isn't in inconf->mta) and send the headers. */
     if ((mta = innconf->mta) == NULL)
-	return "Can't start mailer - mta not set";
+	return "Can't start mailer -- mta not set";
     snprintf(buff, sizeof(buff), innconf->mta, address);
     if ((F = popen(buff, "w")) == NULL)
 	return "Can't start mailer";
@@ -682,13 +672,13 @@ ValidNewsgroups(char *hdr, char **modgroup)
     IsNewgroup = (p && strncmp(p, "newgroup", 8) == 0);
     groups = xstrdup(hdr);
     if ((p = strtok(groups, NGSEPS)) == NULL)
-	return "Can't parse newsgroups line";
+	return "Can't parse Newsgroups: header";
     Error[0] = '\0';
 
-    /* Reject all articles with Approved headers unless the user is allowed to
-       add them, even to unmoderated or local groups.  We want to reject them
-       to unmoderated groups in case there's a disagreement of opinion
-       between various sites as to the moderation status. */
+    /* Reject all articles with Approved: headers unless the user is allowed to
+     * add them, even to unmoderated or local groups.  We want to reject them
+     * to unmoderated groups in case there's a disagreement of opinion
+     * between various sites as to the moderation status. */
     approved = HDR(HDR__APPROVED) != NULL;
     if (approved && !PERMaccessconf->allowapproved) {
         snprintf(Error, sizeof(Error),
@@ -698,8 +688,8 @@ ValidNewsgroups(char *hdr, char **modgroup)
     FoundOne = false;
     h = DDstart((FILE *)NULL, (FILE *)NULL);
     do {
-	if (innconf->mergetogroups && p[0] == 't' && p[1] == 'o' && p[2] == '.')
-	    p = "to";
+	if (innconf->mergetogroups && strncmp(p, "to.", 3) == 0)
+	    p = (char *) "to";
         if (!hookpresent && PERMspecified) {
 	    grplist[0] = p;
 	    grplist[1] = NULL;
@@ -718,12 +708,12 @@ ValidNewsgroups(char *hdr, char **modgroup)
 	if (PY_use_dynamic) {
 	    char    *reply;
 
-	    /* Authorize user using Python module method dynamic */
+	    /* Authorize user using Python module method dynamic. */
 	    if (PY_dynamic(PERMuser, p, true, &reply) < 0) {
-	        syslog(L_NOTICE, "PY_dynamic(): authorization skipped due to no Python dynamic method defined.");
+	        syslog(L_NOTICE, "PY_dynamic(): authorization skipped due to no Python dynamic method defined");
 	    } else {
 	        if (reply != NULL) {
-		    syslog(L_TRACE, "PY_dynamic() returned a refuse string for user %s at %s who wants to read %s: %s", PERMuser, ClientHost, p, reply);
+		    syslog(L_TRACE, "PY_dynamic() returned a refuse string for user %s at %s who wants to read %s: %s", PERMuser, Client.host, p, reply);
 		    snprintf(Error, sizeof(Error), "%s\r\n", reply);
                     free(reply);
 		    break;
@@ -740,14 +730,14 @@ ValidNewsgroups(char *hdr, char **modgroup)
 	case NF_FLAG_NOLOCAL:
 	    if (!PERMaccessconf->locpost)
 		snprintf(Error, sizeof(Error),
-                         "Postings to \"%s\" are not allowed here.", p);
+                         "Postings to \"%s\" are not allowed here", p);
 	    break;
 	case NF_FLAG_EXCLUDED:
 	    /* Do NOT return an error. */
 	    break;
 	case NF_FLAG_ALIAS:
 	    snprintf(Error, sizeof(Error),
-                     "The newsgroup \"%s\" has been renamed.\n", p);
+                     "The newsgroup \"%s\" has been renamed\n", p);
 	    break;
 	}
     } while ((p = strtok((char *)NULL, NGSEPS)) != NULL);
@@ -777,14 +767,14 @@ ValidNewsgroups(char *hdr, char **modgroup)
 
 
 /*
-**  Send a quit message to the server, eat its reply.
+**  Send a QUIT message to the server, eat its reply.
 */
 static void
 SendQuit(FILE *FromServer, FILE *ToServer)
 {
-    char	buff[NNTP_STRLEN];
+    char	buff[NNTP_MAXLEN_COMMAND];
 
-    fprintf(ToServer, "quit\r\n");
+    fprintf(ToServer, "QUIT\r\n");
     fflush(ToServer);
     fclose(ToServer);
     fgets(buff, sizeof buff, FromServer);
@@ -800,7 +790,7 @@ OfferArticle(char *buff, int buffsize, FILE *FromServer, FILE *ToServer)
 {
     static char		CANTSEND[] = "Can't send %s to server, %s";
 
-    fprintf(ToServer, "ihave %s\r\n", HDR(HDR__MESSAGEID));
+    fprintf(ToServer, "IHAVE %s\r\n", HDR(HDR__MESSAGEID));
     if (FLUSH_ERROR(ToServer)
      || fgets(buff, buffsize, FromServer) == NULL) {
 	snprintf(buff, sizeof(buff), CANTSEND, "IHAVE", strerror(errno));
@@ -816,7 +806,7 @@ OfferArticle(char *buff, int buffsize, FILE *FromServer, FILE *ToServer)
 static const char *
 SpoolitTo(char *article, char *err, char *SpoolDir)
 {
-    static char	CANTSPOOL[NNTP_STRLEN+2];
+    static char	CANTSPOOL[NNTP_MAXLEN_COMMAND+2];
     HEADER *hp;
     FILE *F = NULL;
     int	i, fd;
@@ -824,7 +814,7 @@ SpoolitTo(char *article, char *err, char *SpoolDir)
     char *spoolfile = NULL;
     char *q;
 
-    /* Initialize the returned error message */
+    /* Initialize the returned error message. */
     snprintf(CANTSPOOL, sizeof(CANTSPOOL),
              "%s and can't write text to local spool file", err);
 
@@ -832,12 +822,12 @@ SpoolitTo(char *article, char *err, char *SpoolDir)
     tmpspool = concatpath(SpoolDir, ".XXXXXX");
     fd = mkstemp(tmpspool);
     if (fd < 0) {
-        syslog(L_FATAL, "cant create temporary spool file %s %m", tmpspool);
+        syslog(L_FATAL, "can't create temporary spool file %s %m", tmpspool);
         goto fail;
     }
     F = fdopen(fd, "w");
     if (F == NULL) {
-        syslog(L_FATAL, "cant open %s %m", tmpspool);
+        syslog(L_FATAL, "can't open %s %m", tmpspool);
         goto fail;
     }
     fchmod(fileno(F), BATCHFILE_MODE);
@@ -866,14 +856,14 @@ SpoolitTo(char *article, char *err, char *SpoolDir)
     }
     fprintf(F, "\n");
 
-    /* Write the article body */
+    /* Write the article body. */
     i = strlen(article);
     if (fwrite(article, 1, i, F) != (size_t)i) {
         fclose(F);
         goto fail;
     }
 
-    /* Flush and catch any errors */
+    /* Flush and catch any errors. */
     if (fclose(F))
         goto fail;
 
@@ -881,16 +871,16 @@ SpoolitTo(char *article, char *err, char *SpoolDir)
     spoolfile = concatpath(SpoolDir, "XXXXXX");
     fd = mkstemp(spoolfile);
     if (fd < 0) {
-        syslog(L_FATAL, "cant create spool file %s %m", spoolfile);
+        syslog(L_FATAL, "can't create spool file %s %m", spoolfile);
         goto fail;
     }
     close(fd);
     if (rename(tmpspool, spoolfile) < 0) {
-        syslog(L_FATAL, "cant rename %s %s %m", tmpspool, spoolfile);
+        syslog(L_FATAL, "can't rename %s %s %m", tmpspool, spoolfile);
 	goto fail;
     }
 
-    /* Article has been spooled */
+    /* Article has been spooled. */
     free(tmpspool);
     free(spoolfile);
     return NULL;
@@ -903,6 +893,7 @@ SpoolitTo(char *article, char *err, char *SpoolDir)
     return CANTSPOOL;
 }
 
+
 /*
 **  Spool article to temp file.
 */
@@ -911,8 +902,11 @@ Spoolit(char *article, char *err)
 {
     return SpoolitTo(article, err, innconf->pathincoming);
 }
- 
-static char *Towire(char *p) {
+
+
+static char *
+Towire(char *p)
+{
     char	*q, *r, *s;
     int		curlen, len = BIG_BUFFER;
 
@@ -928,7 +922,7 @@ static char *Towire(char *p) {
 		if (*(r - 1) != '\r')
 		    *q++ = '\r';
 	    } else {
-		/* this should not happen */
+		/* This should not happen. */
 		free(s);
 		return NULL;
 	    }
@@ -945,11 +939,12 @@ static char *Towire(char *p) {
     return s;
 }
 
+
+/*
+**  The main function which handles POST and IHAVE.
+*/
 const char *
-ARTpost(char *article,
-	char *idbuff,
-	bool ihave,
-	bool *permanent)
+ARTpost(char *article, char *idbuff, bool ihave, bool *permanent)
 {
     static char	CANTSEND[] = "Can't send %s to server, %s";
     int		i;
@@ -958,15 +953,14 @@ ARTpost(char *article,
     HEADER	*hp;
     FILE	*ToServer;
     FILE	*FromServer;
-    char	buff[NNTP_STRLEN + 2], frombuf[SMBUF];
+    char	buff[NNTP_MAXLEN_COMMAND + 2], frombuf[SMBUF];
     char	*modgroup = NULL;
     const char	*error;
     char	*TrackID;
     char	*DirTrackID;
     FILE	*ftd;
-    char	SDir[255];
 
-    /* Assume errors are permanent, until we discover otherwise */
+    /* Assume errors are permanent, until we discover otherwise. */
     *permanent = true;
 
     /* Set up the other headers list. */
@@ -1030,7 +1024,7 @@ ARTpost(char *article,
     if ((PERMaccessconf->localmaxartsize > 0) &&
 		(strlen(article) > (unsigned)PERMaccessconf->localmaxartsize)) {
         snprintf(Error, sizeof(Error),
-                 "Article is bigger then local limit of %ld bytes\n",
+                 "Article is bigger than local limit of %ld bytes\n",
                  PERMaccessconf->localmaxartsize);
         if (modgroup)
             free(modgroup);
@@ -1038,23 +1032,26 @@ ARTpost(char *article,
     }
 
 #if defined(DO_PERL)
-    /* Calls the Perl subroutine for headers management */
+    /* Calls the Perl subroutine for headers management. */
     p = PERMaccessconf->nnrpdperlfilter ? HandleHeaders(article) : NULL;
     if (p != NULL) {
+        char SDir[255];
+
 	if (idbuff) {
 	    if (modgroup)
-		sprintf(idbuff, "(mailed to moderator for %s)", modgroup);
+		snprintf(idbuff, SMBUF, "(mailed to moderator for %s)",
+                         modgroup);
 	    else
 		strlcpy(idbuff, HDR(HDR__MESSAGEID), SMBUF);
 	}
 	if (strncmp(p, "DROP", 4) == 0) {
-	    syslog(L_NOTICE, "%s post %s", ClientHost, p);
+	    syslog(L_NOTICE, "%s post failed %s", Client.host, p);
 	    if (modgroup)
 		free(modgroup);
 	    return NULL;
 	}
 	else if (strncmp(p, "SPOOL", 5) == 0) {
-	    syslog(L_NOTICE, "%s post %s", ClientHost, p);
+	    syslog(L_NOTICE, "%s post failed %s", Client.host, p);
 	    strlcpy(SDir, innconf->pathincoming, sizeof(SDir));
 	    if (modgroup) {
 		free(modgroup);
@@ -1066,6 +1063,13 @@ ARTpost(char *article,
 		return SpoolitTo(article, p, SDir);
             }
 	}
+        else if (strncmp(p, "CLOSE", 5) == 0) {
+            syslog(L_NOTICE, "%s post failed %s", Client.host, p);
+            Reply("%d NNTP server unavailable; no posting\r\n",
+                  NNTP_FAIL_TERMINATING);
+            POSTrejected++;
+            ExitWithStats(1, true);
+        }
 	else
 	{
 	    if (modgroup)
@@ -1075,13 +1079,13 @@ ARTpost(char *article,
     }
 #endif /* defined(DO_PERL) */
 
-    /* handle mailing to moderated groups */
+    /* Handle mailing to moderated groups. */
 
     if (modgroup) {
 	if (idbuff != NULL) {
 	    const char *retstr;
 	    retstr = MailArticle(modgroup, article);
-	    strcpy (idbuff,"(mailed to moderator)") ;
+	    strlcpy (idbuff, "(mailed to moderator)", SMBUF);
 	    return retstr;
 	}
 	return MailArticle(modgroup, article);
@@ -1098,19 +1102,20 @@ ARTpost(char *article,
 
     /* Open a local connection to the server. */
     if (PERMaccessconf->nnrpdposthost != NULL)
-	i = NNTPconnect(PERMaccessconf->nnrpdposthost, PERMaccessconf->nnrpdpostport,
-					&FromServer, &ToServer, buff);
+	i = NNTPconnect(PERMaccessconf->nnrpdposthost,
+                        PERMaccessconf->nnrpdpostport, &FromServer,
+                        &ToServer, buff, sizeof(buff));
     else {
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
-	i = NNTPlocalopen(&FromServer, &ToServer, buff);
+	i = NNTPlocalopen(&FromServer, &ToServer, buff, sizeof(buff));
 #else
-	i = NNTPremoteopen(innconf->port, &FromServer,
-					&ToServer, buff);
+	i = NNTPremoteopen(innconf->port, &FromServer, &ToServer, buff,
+                           sizeof(buff));
 #endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
     }
 
     /* If we cannot open the connection, initialize the error message and
-     * attempt to recover from this by spooling it locally */
+     * attempt to recover from this by spooling it locally. */
     if (i < 0) {
 	if (buff[0])
 	    strlcpy(Error, buff, sizeof(Error));
@@ -1119,9 +1124,10 @@ ARTpost(char *article,
                      strerror(errno));
         return Spoolit(article,Error);
     }
+
     if (Tracing)
 	syslog(L_TRACE, "%s post_connect %s",
-	    ClientHost, PERMaccessconf->nnrpdposthost ? PERMaccessconf->nnrpdposthost : "localhost");
+	    Client.host, PERMaccessconf->nnrpdposthost ? PERMaccessconf->nnrpdposthost : "localhost");
 
     /* The code below ignores too many return values for my tastes.  At least
      * they are all inside cases that are most likely never going to happen --
@@ -1129,7 +1135,7 @@ ARTpost(char *article,
 
     /* Offer article to server. */
     i = OfferArticle(buff, (int)sizeof buff, FromServer, ToServer);
-    if (i == NNTP_AUTH_NEEDED_VAL) {
+    if (i == NNTP_FAIL_AUTH_NEEDED) {
         /* Send authorization. */
         if (NNTPsendpassword(PERMaccessconf->nnrpdposthost, FromServer, ToServer) < 0) {
             snprintf(Error, sizeof(Error), "Can't authorize with %s",
@@ -1138,18 +1144,18 @@ ARTpost(char *article,
         }
         i = OfferArticle(buff, (int)sizeof buff, FromServer, ToServer);
     }
-    if (i != NNTP_SENDIT_VAL) {
+    if (i != NNTP_CONT_IHAVE) {
         strlcpy(Error, buff, sizeof(Error));
         SendQuit(FromServer, ToServer);
-	if (i != NNTP_HAVEIT_VAL)
+	if (i != NNTP_FAIL_IHAVE_REFUSE)
 	    return Spoolit(article, Error);
-	if (i == NNTP_REJECTIT_VAL || i == NNTP_RESENDIT_VAL) {
+	if (i == NNTP_FAIL_IHAVE_REJECT || i == NNTP_FAIL_IHAVE_DEFER) {
 	    *permanent = false;
 	}
         return Error;
     }
     if (Tracing)
-	syslog(L_TRACE, "%s post starting", ClientHost);
+	syslog(L_TRACE, "%s post starting", Client.host);
 
     /* Write the headers and a blank line. */
     for (hp = Table; hp < ARRAY_END(Table); hp++)
@@ -1157,7 +1163,7 @@ ARTpost(char *article,
             q = xstrndup(hp->Value, hp->Body - hp->Value + hp->Len);
 	    if (strchr(q, '\n') != NULL) {
 		if ((p = Towire(q)) != NULL) {
-		    /* there is no white space, if hp->Value and hp->Body is the same */
+		    /* There is no white space, if hp->Value and hp->Body are the same. */
 		    if (*hp->Value == ' ' || *hp->Value == '\t')
 			fprintf(ToServer, "%s:%s\r\n", hp->Name, p);
 		    else
@@ -1165,7 +1171,7 @@ ARTpost(char *article,
 		    free(p);
 		}
 	    } else {
-		/* there is no white space, if hp->Value and hp->Body is the same */
+		/* There is no white space, if hp->Value and hp->Body are the same. */
 		if (*hp->Value == ' ' || *hp->Value == '\t')
 		    fprintf(ToServer, "%s:%s\r\n", hp->Name, q);
 		else
@@ -1201,22 +1207,22 @@ ARTpost(char *article,
     }
 
     /* Did the server want the article? */
-    if ((i = atoi(buff)) != NNTP_TOOKIT_VAL) {
+    if ((i = atoi(buff)) != NNTP_OK_IHAVE) {
 	strlcpy(Error, buff, sizeof(Error));
 	SendQuit(FromServer, ToServer);
-	syslog(L_TRACE, "%s server rejects %s from %s", ClientHost, HDR(HDR__MESSAGEID), HDR(HDR__PATH));
-	if (i != NNTP_REJECTIT_VAL && i != NNTP_HAVEIT_VAL)
+	syslog(L_TRACE, "%s server rejects %s from %s", Client.host, HDR(HDR__MESSAGEID), HDR(HDR__PATH));
+	if (i != NNTP_FAIL_IHAVE_REJECT && i != NNTP_FAIL_IHAVE_REFUSE)
 	    return Spoolit(article, Error);
-	if (i == NNTP_REJECTIT_VAL || i == NNTP_RESENDIT_VAL) {
+	if (i == NNTP_FAIL_IHAVE_REJECT || i == NNTP_FAIL_IHAVE_DEFER) {
 	    *permanent = false;
 	}
 	return Error;
     }
 
-    /* Send a quit and close down */
+    /* Send a quit and close down. */
     SendQuit(FromServer, ToServer);
 
-    /* Tracking */
+    /* Tracking. */
     if (PERMaccessconf->readertrack) {
         TrackID = concat(innconf->pathlog, "/trackposts/track.",
                          HDR(HDR__MESSAGEID), (char *) 0);
@@ -1227,7 +1233,7 @@ ARTpost(char *article,
 	}
 	if (ftd == NULL && (ftd = fopen(TrackID,"w")) == NULL) {
 	    syslog(L_ERROR, "%s (%s) open %s: %m",
-		ClientHost, Username, TrackID);
+		Client.host, Username, TrackID);
 	    free(TrackID);
 	    return NULL;
 	}
@@ -1236,7 +1242,7 @@ ARTpost(char *article,
                 q = xstrndup(hp->Value, hp->Body - hp->Value + hp->Len);
 		if (strchr(q, '\n') != NULL) {
 		    if ((p = Towire(q)) != NULL) {
-			/* there is no white space, if hp->Value and hp->Body is the same */
+			/* There is no white space, if hp->Value and hp->Body are the same. */
 			if (*hp->Value == ' ' || *hp->Value == '\t')
 			    fprintf(ftd, "%s:%s\r\n", hp->Name, p);
 			else
@@ -1244,7 +1250,7 @@ ARTpost(char *article,
 			free(p);
 		    }
 		} else {
-		    /* there is no white space, if hp->Value and hp->Body is the same */
+		    /* There is no white space, if hp->Value and hp->Body are the same. */
 		    if (*hp->Value == ' ' || *hp->Value == '\t')
 			fprintf(ftd, "%s:%s\r\n", hp->Name, q);
 		    else
@@ -1266,13 +1272,13 @@ ARTpost(char *article,
 	NNTPsendarticle(article, ftd, true);
 	if (fclose(ftd) != EOF) {
 	    syslog(L_NOTICE, "%s (%s) posttrack ok %s",
-		ClientHost, Username, TrackID);
+		Client.host, Username, TrackID);
 	    if (LLOGenable)
 		fprintf(locallog, "%s (%s) posttrack ok %s\n",
-		    ClientHost, Username, TrackID);
+		    Client.host, Username, TrackID);
 	} else {
 	    syslog(L_ERROR, "%s (%s) posttrack error 2 %s",
-		ClientHost, Username, TrackID);
+		Client.host, Username, TrackID);
 	}
 	free(TrackID);
     }
