@@ -1775,15 +1775,6 @@ sub collect($$$$$$) {
     return 1 if $left =~ /^starttls: \S+ with cipher \S+ \(\d+\/\d+ bits\) no authentication$/o;
   }
   ########
-  ## inndstart
-  if ($prog eq "inndstart") {
-    # cant bind Address already in use
-    # cant bind Permission denied
-    return 1 if $left =~ /cant bind /o;
-    # cant setgroups Operation not permitted
-    return 1 if $left =~ /cant setgroups /o;
-  }
-  ########
   ## overchan
   if ($prog eq "overchan") {
     # times
@@ -2003,7 +1994,7 @@ sub collect($$$$$$) {
       }
       return 1;
     }
-    # control_XXgroup, foo.bar [moderated] who who /x/y/12, peer, action, 1
+    # control_XXgroup, foo.bar [moderated] who who token, [pattern], [pattern], encoding, peer, action, 1
     #
     # Various other random junk can end up in the moderated field, like y,
     # unmoderated, m, etc. depending on what the control message says.  It
@@ -2013,23 +2004,23 @@ sub collect($$$$$$) {
                   (\s\S+)?            # optional
                   \s(\S+)             # e-mail
                   \s\S+               # e-mail
-                  \s\S+,              # filename
+                  \s\S+,              # storage token
                   \s(?:\S+)?,         # exclusion pattern
                   \s(?:\S+)?,         # drop pattern
                   \s\S+,              # local encoding
                   \s\S+,              # server
                   \s([^=,]+(?:=\S+)?),            # action
-                  \s*(.*)             # code
+                  \s*(.*)             # 1 if message approved and first logged in the file
                   /x) {
       if ($1 eq 'newgroup') {
 	$controlchan_new{$3}++;
       } elsif ($1 eq 'rmgroup') {
 	$controlchan_rm{$3}++;
       } else {
-	$controlchan_other{$3}++;
+	$controlchan_other{$3}++ if $5 >= 0;
       }
       $controlchan_who{$3}++;
-      $controlchan_ok{$3} += $5;
+      $controlchan_ok{$3} += $5 if $5 > 0;
       my $action = $4;
       my $email = $3;
       $action =~ s/=.*//;
