@@ -1,4 +1,4 @@
-/*  $Id: interface.c,v 1.70 2003/03/27 23:56:00 rra Exp $
+/*  $Id: interface.c 7609 2007-07-22 16:35:51Z eagle $
 **
 **  Storage Manager interface
 */
@@ -6,16 +6,16 @@
 #include "clibrary.h"
 #include <ctype.h>
 #include <errno.h>
-#include <syslog.h>
 #include <time.h>
 
 #include "conffile.h"
 #include "inn/innconf.h"
+#include "inn/messages.h"
 #include "inn/wire.h"
 #include "interface.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 #include "methods.h"
-#include "paths.h"
+#include "inn/paths.h"
 
 typedef enum {INIT_NO, INIT_DONE, INIT_FAIL} INITTYPE;
 typedef struct {
@@ -31,7 +31,6 @@ static STORAGE_SUB      *subscriptions = NULL;
 static unsigned int     typetoindex[256];
 int                     SMerrno;
 char                    *SMerrorstr = NULL;
-static bool             ErrorAlloc = false;
 static bool             Initialized = false;
 bool			SMopenmode = false;
 bool			SMpreopen = false;
@@ -116,107 +115,6 @@ TOKEN TextToToken(const char *text) {
     return token;
 }
 
-/*
-** Given an article and length in non-wire format, return a malloced region
-** containing the article in wire format.  Set *newlen to the length of the
-** new article.
-*/ 
-char *
-ToWireFmt(const char *article, size_t len, size_t *newlen)
-{
-    size_t bytes;
-    char *newart;
-    const char *p;
-    char  *dest;
-    bool atstartofline=true;
-
-    /* First go thru article and count number of bytes we need. */
-    for (bytes = 0, p=article ; p < &article[len] ; ++p) {
-	if (*p == '.' && atstartofline) ++bytes; /* 1 byte for escaping . */
-	++bytes;
-	if (*p == '\n') {
-	    ++bytes; /* need another byte for CR */
-	    atstartofline = true; /* next char starts new line */
-	} else {
-	    atstartofline = false;
-	}
-    }
-    bytes += 3; /* for .\r\n */
-    newart = xmalloc(bytes + 1);
-    *newlen = bytes;
-
-    /* now copy the article, making changes */
-    atstartofline = true;
-    for (p=article, dest=newart ; p < &article[len] ; ++p) {
-	if (*p == '\n') {
-	    *dest++ = '\r';
-	    *dest++ = '\n';
-	    atstartofline = true;
-	} else {
-	    if (atstartofline && *p == '.') *dest++ = '.'; /* add extra . */
-	    *dest++ = *p;
-	    atstartofline = false;
-	}
-    }
-    *dest++ = '.';
-    *dest++ = '\r';
-    *dest++ = '\n';
-    *dest = '\0';
-    return newart;
-}
-
-char *
-FromWireFmt(const char *article, size_t len, size_t *newlen)
-{
-    size_t bytes;
-    char *newart;
-    const char *p;
-    char *dest;
-    bool atstartofline = true;
-
-    /* First go thru article and count number of bytes we need */
-    for (bytes = 0, p=article ; p < &article[len] ; ) {
-	/* check for terminating .\r\n and if so break */
-	if (p == &article[len-3] && *p == '.' && p[1] == '\r' && p[2] == '\n')
-	    break;
-	/* check for .. at start-of-line */
-	if (atstartofline && p < &article[len-1] && *p == '.' && p[1] == '.') {
-	    bytes++; /* only output 1 byte */
-	    p+=2; 
-	    atstartofline = false;
-	} else if (p < &article[len-1] && *p == '\r' && p[1] == '\n') { 
-	    bytes++; /* \r\n counts as only one byte in output */
-	    p += 2;
-	    atstartofline = true;
-	} else {
-	    bytes++;
-	    p++;
-	    atstartofline = false;
-	}
-    }
-    newart = xmalloc(bytes + 1);
-    *newlen = bytes;
-    for (p = article, dest = newart ; p < &article[len]; ) {
-	/* check for terminating .\r\n and if so break */
-	if (p == &article[len-3] && *p == '.' && p[1] == '\r' && p[2] == '\n')
-	    break;
-	if (atstartofline && p < &article[len-1] && *p == '.' && p[1] == '.') {
-	    *dest++ = '.';
-	    p += 2;
-	    atstartofline = false;
-	} else if (p < &article[len-1] && *p == '\r' && p[1] == '\n') {
-	    *dest++ = '\n';
-	    p += 2;
-	    atstartofline = true;
-	} else {
-	    *dest++ = *p++;
-	    atstartofline = false;
-	}
-    }
-    *dest = '\0';
-    return newart;
-}
-
 /*
 **  get Xref header without pathhost
 */
@@ -336,16 +234,16 @@ static time_t ParseTime(char *tmbuf)
 #define SMexactmatch 16
 
 static CONFTOKEN smtoks[] = {
-  { SMlbrace,	"{" },
-  { SMrbrace,	"}" },
-  { SMmethod,	"method" },
-  { SMgroups,	"newsgroups:" },
-  { SMsize,	"size:" },
-  { SMclass,	"class:" },
-  { SMexpire,	"expires:" },
-  { SMoptions,	"options:" },
-  { SMexactmatch,	"exactmatch:" },
-  { 0, 0 }
+    { SMlbrace,         (char *) "{"            },
+    { SMrbrace,         (char *) "}"            },
+    { SMmethod,         (char *) "method"       },
+    { SMgroups,         (char *) "newsgroups:"  },
+    { SMsize,           (char *) "size:"        },
+    { SMclass,          (char *) "class:"       },
+    { SMexpire,         (char *) "expires:"     },
+    { SMoptions,        (char *) "options:"     },
+    { SMexactmatch,     (char *) "exactmatch:"  },
+    { 0,                NULL                    }
 };
 
 /* Open the config file and parse it, generating the policy data */
@@ -384,11 +282,11 @@ SMreadconfig(void)
 	method_data[i].initialized = INIT_NO;
 	method_data[i].configured = false;
     }
-    path = concatpath(innconf->pathetc, _PATH_STORAGECTL);
+    path = concatpath(innconf->pathetc, INN_PATH_STORAGECTL);
     f = CONFfopen(path);
     if (f == NULL) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	syslog(L_ERROR, "SM Could not open %s: %m", path);
+        syswarn("SM: cant open %s", path);
         free(path);
 	return false;
     }
@@ -399,18 +297,18 @@ SMreadconfig(void)
 	if (!inbrace) {
 	    if (tok->type != SMmethod) {
 		SMseterror(SMERR_CONFIG, "Expected 'method' keyword");
-		syslog(L_ERROR, "SM expected 'method' keyword, line %d", f->lineno);
+		warn("SM: expected 'method' keyword, line %d", f->lineno);
 		return false;
 	    }
 	    if ((tok = CONFgettoken(0, f)) == NULL) {
 		SMseterror(SMERR_CONFIG, "Expected method name");
-		syslog(L_ERROR, "SM expected method name, line %d", f->lineno);
+		warn("SM: expected method name, line %d", f->lineno);
 		return false;
 	    }
 	    method = xstrdup(tok->name);
 	    if ((tok = CONFgettoken(smtoks, f)) == NULL || tok->type != SMlbrace) {
 		SMseterror(SMERR_CONFIG, "Expected '{'");
-		syslog(L_ERROR, "SM Expected '{', line %d", f->lineno);
+		warn("SM: Expected '{', line %d", f->lineno);
 		return false;
 	    }
 	    inbrace = 1;
@@ -431,7 +329,7 @@ SMreadconfig(void)
 	    else {
 		if ((tok = CONFgettoken(0, f)) == NULL) {
 		    SMseterror(SMERR_CONFIG, "Keyword with no value");
-		    syslog(L_ERROR, "SM keyword with no value, line %d", f->lineno);
+		    warn("SM: keyword with no value, line %d", f->lineno);
 		    return false;
 		}
 		p = tok->name;
@@ -450,6 +348,12 @@ SMreadconfig(void)
 		    break;
 		  case SMclass:
 		    class = atoi(p);
+                    if (class > NUM_STORAGE_CLASSES) {
+                        SMseterror(SMERR_CONFIG, "Storage class too large");
+                        warn("SM: storage class larger than %d, line %d",
+                             NUM_STORAGE_CLASSES, f->lineno);
+                        return false;
+                    }
 		    break;
 		  case SMexpire:
 		    q = strchr(p, ',');
@@ -471,8 +375,10 @@ SMreadconfig(void)
 			exactmatch = true;
 		    break;
 		  default:
-		    SMseterror(SMERR_CONFIG, "Unknown keyword in method declaration");
-		    syslog(L_ERROR, "SM Unknown keyword in method declaration, line %d: %s", f->lineno, tok->name);
+		    SMseterror(SMERR_CONFIG,
+                               "Unknown keyword in method declaration");
+		    warn("SM: Unknown keyword in method declaration, line %d:"
+                         " %s", f->lineno, tok->name);
 		    free(method);
 		    return false;
 		    break;
@@ -492,14 +398,15 @@ SMreadconfig(void)
 	    }
 	    if (sub->type == TOKEN_EMPTY) {
 		SMseterror(SMERR_CONFIG, "Invalid storage method name");
-		syslog(L_ERROR, "SM no configured storage methods are named '%s'", method);
+		warn("SM: no configured storage methods are named '%s'",
+                     method);
 		free(options);
 		free(sub);
 		return false;
 	    }
 	    if (!pattern) {
 		SMseterror(SMERR_CONFIG, "pattern not defined");
-		syslog(L_ERROR, "SM no pattern defined");
+                warn("SM: no pattern defined");
 		free(options);
 		free(sub);
 		return false;
@@ -580,7 +487,8 @@ bool SMinit(void) {
 		method_data[i].initialized = INIT_FAIL;
 		method_data[i].selfexpire = false;
 		method_data[i].expensivestat = true;
-		syslog(L_ERROR, "SM storage method '%s' failed initialization", storage_methods[i].name);
+                warn("SM: storage method '%s' failed initialization",
+                     storage_methods[i].name);
 		allok = false;
 	    }
 	}
@@ -589,8 +497,9 @@ bool SMinit(void) {
     if (!allok) {
 	SMshutdown();
 	Initialized = false;
-	SMseterror(SMERR_UNDEFINED, "one or more storage methods failed initialization");
-	syslog(L_ERROR, "SM one or more storage methods failed initialization");
+	SMseterror(SMERR_UNDEFINED,
+                   "one or more storage methods failed initialization");
+        warn("SM: one or more storage methods failed initialization");
 	return false;
     }
     if (!once && atexit(SMshutdown) < 0) {
@@ -621,14 +530,15 @@ static bool InitMethod(STORAGETYPE method) {
 
     if (!method_data[method].configured) {
 	method_data[method].initialized = INIT_FAIL;
-	SMseterror(SMERR_UNDEFINED, "storage method is not configured.");
+	SMseterror(SMERR_UNDEFINED, "storage method is not configured");
 	return false;
     }
     if (!storage_methods[method].init(&smattr)) {
 	method_data[method].initialized = INIT_FAIL;
 	method_data[method].selfexpire = false;
 	method_data[method].expensivestat = true;
-	SMseterror(SMERR_UNDEFINED, "Could not initialize storage method late.");
+	SMseterror(SMERR_UNDEFINED,
+                   "Could not initialize storage method late");
 	return false;
     }
     method_data[method].initialized = INIT_DONE;
@@ -709,6 +619,7 @@ TOKEN SMstore(const ARTHANDLE article) {
     TOKEN               result;
 
     if (!SMopenmode) {
+	memset(&result, 0, sizeof(result));
 	result.type = TOKEN_EMPTY;
 	SMseterror(SMERR_INTERNAL, "read only storage api");
 	return result;
@@ -728,8 +639,8 @@ ARTHANDLE *SMretrieve(const TOKEN token, const RETRTYPE amount) {
 	return NULL;
     }
     if (method_data[typetoindex[token.type]].initialized == INIT_NO && !InitMethod(typetoindex[token.type])) {
-	syslog(L_ERROR, "SM could not find token type or method was not initialized (%d)",
-	       token.type);
+        warn("SM: could not find token type or method was not initialized"
+             " (%d)", token.type);
 	SMseterror(SMERR_UNINIT, NULL);
 	return NULL;
     }
@@ -740,7 +651,9 @@ ARTHANDLE *SMretrieve(const TOKEN token, const RETRTYPE amount) {
 
 }
 
-ARTHANDLE *SMnext(const ARTHANDLE *article, const RETRTYPE amount) {
+ARTHANDLE *
+SMnext(ARTHANDLE *article, const RETRTYPE amount)
+{
     unsigned char       i;
     int                 start;
     ARTHANDLE           *newart;
@@ -776,7 +689,7 @@ void SMfreearticle(ARTHANDLE *article) {
 	return;
     }
     if (method_data[typetoindex[article->type]].initialized == INIT_NO && !InitMethod(typetoindex[article->type])) {
-	syslog(L_ERROR, "SM can't free article with uninitialized method");
+	warn("SM: can't free article with uninitialized method");
 	return;
     }
     storage_methods[typetoindex[article->type]].freearticle(article);
@@ -793,7 +706,7 @@ bool SMcancel(TOKEN token) {
     }
     if (method_data[typetoindex[token.type]].initialized == INIT_NO && !InitMethod(typetoindex[token.type])) {
 	SMseterror(SMERR_UNINIT, NULL);
-	syslog(L_ERROR, "SM can't cancel article with uninitialized method");
+	warn("SM: can't cancel article with uninitialized method");
 	return false;
     }
     return storage_methods[typetoindex[token.type]].cancel(token);
@@ -813,7 +726,7 @@ bool SMprobe(PROBETYPE type, TOKEN *token, void *value) {
 	}
 	if (method_data[typetoindex[token->type]].initialized == INIT_NO && !InitMethod(typetoindex[token->type])) {
 	    SMseterror(SMERR_UNINIT, NULL);
-	    syslog(L_ERROR, "SM can't cancel article with uninitialized method");
+	    warn("SM: can't probe article with uninitialized method");
 	    return false;
 	}
 	if ((ann = (struct artngnum *)value) == NULL)
@@ -861,7 +774,8 @@ bool SMflushcacheddata(FLUSHTYPE type) {
     for (i = 0; i < NUM_STORAGE_METHODS; i++) {
 	if (method_data[i].initialized == INIT_DONE &&
 	    !storage_methods[i].flushcacheddata(type))
-	    syslog(L_ERROR, "SM can't flush cached data method '%s'", storage_methods[i].name);
+	    warn("SM: can't flush cached data method '%s'",
+                 storage_methods[i].name);
     }
     return true;
 }
@@ -872,7 +786,7 @@ void SMprintfiles(FILE *file, TOKEN token, char **xref, int ngroups) {
     if (method_data[typetoindex[token.type]].initialized == INIT_NO
         && !InitMethod(typetoindex[token.type])) {
 	SMseterror(SMERR_UNINIT, NULL);
-	syslog(L_ERROR, "SM can't print files for article with uninitialized method");
+	warn("SM: can't print files for article with uninitialized method");
 	return;
     }
     storage_methods[typetoindex[token.type]].printfiles(file, token, xref, ngroups);
@@ -901,56 +815,51 @@ void SMshutdown(void) {
     Initialized = false;
 }
 
-void SMseterror(int errornum, char *error) {
-    if (ErrorAlloc)
-	free(SMerrorstr);
-
-    ErrorAlloc = false;
+void
+SMseterror(int errornum, const char *error)
+{
+    if (SMerrorstr != NULL)
+        free(SMerrorstr);
     
-    if ((errornum == SMERR_UNDEFINED) && (errno == ENOENT))
+    if (errornum == SMERR_UNDEFINED && errno == ENOENT)
 	errornum = SMERR_NOENT;
-	    
     SMerrno = errornum;
 
     if (error == NULL) {
 	switch (SMerrno) {
 	case SMERR_UNDEFINED:
-	    SMerrorstr = xstrdup(strerror(errno));
-	    ErrorAlloc = true;
+            error = strerror(errno);
 	    break;
 	case SMERR_INTERNAL:
-	    SMerrorstr = "Internal error";
+	    error = "Internal error";
 	    break;
 	case SMERR_NOENT:
-	    SMerrorstr = "Token not found";
+	    error = "Token not found";
 	    break;
 	case SMERR_TOKENSHORT:
-	    SMerrorstr = "Configured token size too small";
+	    error = "Configured token size too small";
 	    break;
 	case SMERR_NOBODY:
-	    SMerrorstr = "No article body found";
+	    error = "No article body found";
 	    break;
 	case SMERR_UNINIT:
-	    SMerrorstr = "Storage manager is not initialized";
+	    error = "Storage manager is not initialized";
 	    break;
 	case SMERR_CONFIG:
-	    SMerrorstr = "Error reading config file";
+	    error = "Error reading config file";
 	    break;
 	case SMERR_BADHANDLE:
-	    SMerrorstr = "Bad article handle";
+	    error = "Bad article handle";
 	    break;
 	case SMERR_BADTOKEN:
-	    SMerrorstr = "Bad token";
+	    error = "Bad token";
 	    break;
 	case SMERR_NOMATCH:
-	    SMerrorstr = "No matching entry in storage.conf";
+	    error = "No matching entry in storage.conf";
 	    break;
 	default:
-	    SMerrorstr = "Undefined error";
+	    error = "Undefined error";
 	}
-    } else {
-	SMerrorstr = xstrdup(error);
-	ErrorAlloc = true;
     }
+    SMerrorstr = xstrdup(error);
 }
-
