@@ -1,4 +1,4 @@
-/*  $Id: tdx-cache.c 8993 2010-03-14 20:41:19Z iulius $
+/*  $Id: tdx-cache.c 7585 2006-11-21 09:37:51Z eagle $
 **
 **  Cache functions for open overview data files.
 **
@@ -197,12 +197,9 @@ tdx_cache_insert(struct cache *cache, HASH hash, struct group_data *data)
 void
 tdx_cache_delete(struct cache *cache, HASH hash)
 {
-    if (!hash_delete(cache->hashtable, &hash)) {
+    if (!hash_delete(cache->hashtable, &hash))
         warn("tradindexed: unable to remove cache entry for %s",
              HashToText(hash));
-    } else {
-        cache->count--;
-    }
 }
 
 
