@@ -1,4 +1,4 @@
-/*  $Id: tdx-data.c 7598 2007-02-09 02:40:51Z eagle $
+/*  $Id: tdx-data.c 8481 2009-05-20 20:10:04Z iulius $
 **
 **  Overview data file handling for the tradindexed overview method.
 **
@@ -26,10 +26,10 @@
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/mmap.h"
-#include "libinn.h"
-#include "ov.h"
+#include "inn/libinn.h"
+#include "inn/ov.h"
 #include "ovinterface.h"
-#include "storage.h"
+#include "inn/storage.h"
 #include "tdx-private.h"
 #include "tdx-structure.h"
 
@@ -215,6 +215,7 @@ tdx_data_new(const char *group, bool writable)
     data = xmalloc(sizeof(struct group_data));
     data->path = group_path(group);
     data->writable = writable;
+    data->remapoutoforder = false;
     data->high = 0;
     data->base = 0;
     data->indexfd = -1;
@@ -458,11 +459,16 @@ tdx_search_open(struct group_data *data, ARTNUM start, ARTNUM end, ARTNUM high)
     if (end < start)
         return NULL;
 
+    if ((end > data->high && high > data->high) || data->remapoutoforder) {
+        data->remapoutoforder = false;
+        unmap_data(data);
+    }
     if (end > data->high && high > data->high) {
         unmap_index(data);
         map_index(data);
         data->high = high;
     }
+
     if (start > data->high)
         return NULL;
 
@@ -512,19 +518,19 @@ tdx_search(struct search *search, struct article *artdata)
     if (search->current > search->limit || search->current > max)
         return false;
 
-    /* Make sure that the offset into the data file is sensible, and try
-       remapping the data file if the portion the offset is pointing to isn't
-       currently mapped.  Otherwise, warn about possible corruption and return
-       a miss. */
+    /* There is a small chance that remapping the data file could make this
+       offset accessible, but changing the memory location in the middle of
+       a search conflicts with what we return for OVSTATICSEARCH.  And it
+       seems not to be an issue in limited testing, although write caching
+       that leads to on-disk IDX and DAT being out of sync could trigger a
+       problem here. */
     if (entry->offset + entry->length > search->data->datalen) {
-        unmap_data(search->data);
-        if (!map_data(search->data))
-            return false;
-    }
-    if (entry->offset + entry->length > search->data->datalen) {
-        warn("Invalid entry for article %lu in %s.IDX: offset %lu length %lu",
+        search->data->remapoutoforder = true;
+        warn("Invalid or inaccessible entry for article %lu in %s.IDX:"
+             " offset %lu length %lu datalength %lu",
              search->current + search->data->base, search->data->path,
-             (unsigned long) entry->offset, (unsigned long) entry->length);
+             (unsigned long) entry->offset, (unsigned long) entry->length,
+             (unsigned long) search->data->datalen);
         return false;
     }
 
@@ -561,7 +567,7 @@ tdx_search_close(struct search *search)
 **  constantly repack if the article numbers are clustered around a particular
 **  value but don't come in order.
 */
-ARTNUM
+static ARTNUM
 index_base(ARTNUM artnum)
 {
     return (artnum > 128) ? (artnum - 128) : 1;
@@ -621,6 +627,31 @@ tdx_data_store(struct group_data *data, const struct article *article)
 }
 
 
+/*
+**  Cancels a particular article by removing its index entry.  The data will
+**  still be present in the data file until the next expire run, but it won't
+**  be returned by searches.
+*/
+bool
+tdx_data_cancel(struct group_data *data, ARTNUM artnum)
+{
+    static const struct index_entry empty;
+    off_t offset;
+
+    if (!data->writable)
+        return false;
+    if (data->base == 0 || artnum < data->base || artnum > data->high)
+        return false;
+    offset = (artnum - data->base) * sizeof(struct index_entry);
+    if (xpwrite(data->indexfd, &empty, sizeof(empty), offset) < 0) {
+        syswarn("tradindexed: cannot cancel index record for %lu in %s.IDX",
+                artnum, data->path);
+        return false;
+    }
+    return true;
+}
+
+
 /*
 **  Start the process of packing a group (rewriting its index file so that it
 **  uses a different article base).  Takes the article number of an article
@@ -1083,7 +1114,7 @@ tdx_data_audit(const char *group, struct group_entry *index, bool fix)
     /* All done.  Close things down and flush the data we changed, if
        necessary. */
     if (changed)
-        inn_mapcntl(index, sizeof(*index), MS_ASYNC);
+        inn_msync_page(index, sizeof(*index), MS_ASYNC);
 
  end:
     tdx_data_close(data);
