@@ -1,4 +1,4 @@
-/* $Id: wire-t.c 10308 2018-12-02 14:33:59Z iulius $ */
+/* $Id: wire-t.c 7585 2006-11-21 09:37:51Z eagle $ */
 /* wire test suite. */
 
 #include "config.h"
@@ -9,7 +9,7 @@
 #include "inn/messages.h"
 #include "inn/wire.h"
 #include "inn/libinn.h"
-#include "tap/basic.h"
+#include "libtest.h"
 
 /* Read in a file and return the contents in newly allocated memory.  Fills in
    the provided stat buffer. */
@@ -60,19 +60,12 @@ main(void)
     ok(9, wire_findbody(p, 1) == NULL);
     ok(10, wire_findbody(p, 0) == NULL);
 
-    if (access("../data/articles/wire-strange", F_OK) == 0) {
-        if (chdir("../data") < 0) {
-            sysbail("cannot chdir to ../data");
-        }
-    } else if (access("data/articles/wire-strange", F_OK) == 0) {
-        if (chdir("data") < 0) {
-            sysbail("cannot chdir to data");
-        }
-    } else if (access("tests/data/articles/wire-strange", F_OK) == 0) {
-        if (chdir("tests/data") < 0) {
-            sysbail("cannot chdir to tests/data");
-        }
-    }
+    if (access("../data/articles/wire-strange", F_OK) == 0)
+        chdir("../data");
+    else if (access("data/articles/wire-strange", F_OK) == 0)
+        chdir("data");
+    else if (access("tests/data/articles/wire-strange", F_OK) == 0)
+        chdir("tests/data");
     article = read_file("articles/wire-strange", &st);
 
     p = wire_findbody(article, st.st_size);
@@ -81,38 +74,38 @@ main(void)
     ok(12, strncmp(p, "Second: Not", strlen("Second: Not")) == 0);
     p = wire_nextline(p, article + st.st_size - 1);
     ok(13, p == NULL);
-    p = wire_findheader(article, st.st_size, "Path", true);
+    p = wire_findheader(article, st.st_size, "Path");
     ok(14, p == article + 6);
-    p = wire_findheader(article, st.st_size, "From", true);
+    p = wire_findheader(article, st.st_size, "From");
     ok(15, strncmp(p, "This is the real", strlen("This is the real")) == 0);
-    p = wire_findheader(article, st.st_size, "SUMMARY", true);
+    p = wire_findheader(article, st.st_size, "SUMMARY");
     ok(16, strncmp(p, "First text", strlen("First text")) == 0);
-    p = wire_findheader(article, st.st_size, "Header", true);
+    p = wire_findheader(article, st.st_size, "Header");
     ok(17, strncmp(p, "This one is real", strlen("This one is real")) == 0);
-    p = wire_findheader(article, st.st_size, "message-id", true);
+    p = wire_findheader(article, st.st_size, "message-id");
     ok(18, strncmp(p, "<foo@example.com>", strlen("<foo@example.com>")) == 0);
-    p = wire_findheader(article, st.st_size, "Second", true);
+    p = wire_findheader(article, st.st_size, "Second");
     ok(19, p == NULL);
-    p = wire_findheader(article, st.st_size, "suBJect", true);
+    p = wire_findheader(article, st.st_size, "suBJect");
     ok(20, strncmp(p, "This is\rnot", strlen("This is\rnot")) == 0);
     end = wire_endheader(p, article + st.st_size - 1);
     ok(21, strncmp(end, "\nFrom: This is", strlen("\nFrom: This is")) == 0);
-    p = wire_findheader(article, st.st_size, "keywordS", true);
+    p = wire_findheader(article, st.st_size, "keywordS");
     ok(22, strncmp(p, "this is --", strlen("this is --")) == 0);
     end = wire_endheader(p, article + st.st_size - 1);
     ok(23, strncmp(end, "\nSummary: ", strlen("\nSummary: ")) == 0);
-    p = wire_findheader(article, st.st_size, "strange", true);
+    p = wire_findheader(article, st.st_size, "strange");
     ok(24, strncmp(p, "This is\n\nnot", strlen("This is\n\nnot")) == 0);
     end = wire_endheader(p, article + st.st_size - 1);
     ok(25, strncmp(end, "\nMessage-ID: ", strlen("\nMessage-ID: ")) == 0);
-    p = wire_findheader(article, st.st_size, "Message", true);
+    p = wire_findheader(article, st.st_size, "Message");
     ok(26, p == NULL);
 
     free(article);
     article = read_file("articles/wire-no-body", &st);
 
     ok(27, wire_findbody(article, st.st_size) == NULL);
-    p = wire_findheader(article, st.st_size, "message-id", true);
+    p = wire_findheader(article, st.st_size, "message-id");
     ok(28, strncmp(p, "<bad-body@example.com>\r\n",
                    strlen("<bad-body@example.com>\r\n")) == 0);
     end = wire_endheader(p, article + st.st_size - 1);
@@ -123,7 +116,7 @@ main(void)
     article = read_file("articles/wire-truncated", &st);
 
     ok(31, wire_findbody(article, st.st_size) == NULL);
-    p = wire_findheader(article, st.st_size, "date", true);
+    p = wire_findheader(article, st.st_size, "date");
     ok(32, strncmp(p, "Mon, 23 Dec", strlen("Mon, 23 Dec")) == 0);
     ok(33, wire_endheader(p, article + st.st_size - 1) == NULL);
     ok(34, wire_nextline(p, article + st.st_size - 1) == NULL);
